@@ -36,11 +36,14 @@ extern "C"
 #define ESPEAK_NG_API
 #endif
 
+#define ESPEAKNG_DEFAULT_VOICE "en"
+
 typedef enum {
 	ENS_GROUP_MASK               = 0x70000000,
 	ENS_GROUP_ERRNO              = 0x00000000, /* Values 0-255 map to errno error codes. */
 	ENS_GROUP_ESPEAK_NG          = 0x10000000, /* eSpeak NG error codes. */
 
+	/* eSpeak NG 1.49.0 */
 	ENS_OK                       = 0,
 	ENS_COMPILE_ERROR            = 0x100001FF,
 	ENS_VERSION_MISMATCH         = 0x100002FF,
@@ -56,6 +59,10 @@ typedef enum {
 	ENS_NO_SPECT_FRAMES          = 0x10000CFF,
 	ENS_EMPTY_PHONEME_MANIFEST   = 0x10000DFF,
 	ENS_SPEECH_STOPPED           = 0x10000EFF,
+
+	/* eSpeak NG 1.49.2 */
+	ENS_UNKNOWN_PHONEME_FEATURE  = 0x10000FFF,
+	ENS_UNKNOWN_TEXT_ENCODING    = 0x100010FF,
 } espeak_ng_STATUS;
 
 typedef enum {
@@ -63,6 +70,13 @@ typedef enum {
 	ENOUTPUT_MODE_SPEAK_AUDIO = 0x0002,
 } espeak_ng_OUTPUT_MODE;
 
+typedef enum {
+	ENGENDER_UNKNOWN = 0,
+	ENGENDER_MALE = 1,
+	ENGENDER_FEMALE = 2,
+	ENGENDER_NEUTRAL = 3,
+} espeak_ng_VOICE_GENDER;
+
 /* eSpeak NG 1.49.0 */
 
 typedef struct espeak_ng_ERROR_CONTEXT_ *espeak_ng_ERROR_CONTEXT;
@@ -105,6 +119,9 @@ espeak_ng_SetPunctuationList(const wchar_t *punctlist);
 ESPEAK_NG_API espeak_ng_STATUS
 espeak_ng_SetVoiceByName(const char *name);
 
+ESPEAK_NG_API espeak_ng_STATUS
+espeak_ng_SetVoiceByFile(const char *filename);
+
 ESPEAK_NG_API espeak_ng_STATUS
 espeak_ng_SetVoiceByProperties(espeak_VOICE *voice_selector);
 
