@@ -38,7 +38,7 @@
 #define ESPEAK_API
 #endif
 
-#define ESPEAK_API_REVISION  10
+#define ESPEAK_API_REVISION  12
 /*
 Revision 2
    Added parameter "options" to eSpeakInitialize()
@@ -68,6 +68,12 @@ Revision 9  30.May.2013
 Revision 10 29.Aug.2014
   Changed phonememode parameter to espeak_TextToPhonemes() and espeak_SetPhonemeTrace
 
+Revision 11 (espeak-ng)
+  Made ESPEAK_API import/export symbols correctly on Windows.
+
+Revision 12 (espeak-ng)
+  Exposed espeak_SetPhonemeCallback. This is available in eSpeak, but was not exposed in this header.
+
 */
          /********************/
          /*  Initialization  */
@@ -194,7 +200,7 @@ ESPEAK_API int espeak_Initialize(espeak_AUDIO_OUTPUT output, int buflength, cons
    output: the audio data can either be played by eSpeak or passed back by the SynthCallback function.
 
    buflength:  The length in mS of sound buffers passed to the SynthCallback function.
-            Value=0 gives a default of 200mS.
+            Value=0 gives a default of 60mS.
             This paramater is only used for AUDIO_OUTPUT_RETRIEVAL and AUDIO_OUTPUT_SYNCHRONOUS modes.
 
    path: The directory which contains the espeak-ng-data directory, or NULL for the default location.
@@ -260,6 +266,11 @@ int UriCallback(int type, const char *uri, const char *base);
              occurs.  The calling program can then play the sound at that point.
 */
 
+#ifdef __cplusplus
+extern "C"
+#endif
+ESPEAK_API void espeak_SetPhonemeCallback(int (*PhonemeCallback)(const char *));
+
 
          /********************/
          /*    Synthesis     */
@@ -318,7 +329,7 @@ ESPEAK_API espeak_ERROR espeak_Synth(const void *text,
 
       espeakSSML   Elements within < > are treated as SSML elements, or if not recognised are ignored.
 
-      espeakPHONEMES  Text within [[ ]] is treated as phonemes codes (in espeak's Hirshenbaum encoding).
+      espeakPHONEMES  Text within [[ ]] is treated as phonemes codes (in espeak's Kirshenbaum encoding).
 
       espeakENDPAUSE  If set then a sentence pause is added at the end of the text.  If not set then
          this pause is suppressed.
@@ -595,6 +606,19 @@ ESPEAK_API const espeak_VOICE **espeak_ListVoices(espeak_VOICE *voice_spec);
    are listed, and they are listed in preference order.
 */
 
+#ifdef __cplusplus
+extern "C"
+#endif
+ESPEAK_API espeak_ERROR espeak_SetVoiceByFile(const char *filename);
+/* Loads a voice given the file path.  Language is not considered.
+   "filename" is a UTF8 string.
+
+   Return: EE_OK: operation achieved
+           EE_BUFFER_FULL: the command can not be buffered;
+             you may try after a while to call the function again.
+	   EE_INTERNAL_ERROR.
+*/
+
 #ifdef __cplusplus
 extern "C"
 #endif
