@@ -33,13 +33,21 @@
 
 #include <espeak-ng/espeak_ng.h>
 #include <espeak-ng/speak_lib.h>
+#include <espeak-ng/encoding.h>
+
+#include "readclause.h"
+#include "synthdata.h"
+#include "wavegen.h"
 
 #include "error.h"
-#include "speech.h"
 #include "phoneme.h"
-#include "synthesize.h"
 #include "voice.h"
+#include "synthesize.h"
 #include "spect.h"
+#include "translate.h"
+#include "dictionary.h"
+
+#define N_ITEM_STRING 256
 
 typedef struct {
 	unsigned int value;
@@ -50,9 +58,6 @@ NAMETAB *manifest = NULL;
 int n_manifest;
 char phsrc[sizeof(path_home)+40]; // Source: path to the 'phonemes' source file.
 
-extern ESPEAK_NG_API int utf8_in(int *c, const char *buf);
-extern int utf8_out(unsigned int c, char *buf);
-
 typedef struct {
 	const char *mnem;
 	int type;
@@ -71,7 +76,6 @@ enum {
 	tPHONEME_TYPE = 1,
 	tPHONEME_FLAG,
 	tTRANSITION,
-	tPLACE,
 	tSTATEMENT,
 	tINSTRN1,
 	tWHICH_PHONEME,
@@ -104,43 +108,39 @@ static keywtab_t k_conditions[] = {
 };
 
 static keywtab_t k_properties[] = {
-	{ "isPause",      0, phPAUSE },
-	{ "isVowel",      0, phVOWEL },
-	{ "isNasal",      0, phNASAL },
-	{ "isLiquid",     0, phLIQUID },
-	{ "isUStop",      0, phSTOP },
-	{ "isVStop",      0, phVSTOP },
-	{ "isVFricative", 0, phVFRICATIVE },
+	{ "isPause",      0, CONDITION_IS_PHONEME_TYPE | phPAUSE },
+	{ "isVowel",      0, CONDITION_IS_PHONEME_TYPE | phVOWEL },
+	{ "isNasal",      0, CONDITION_IS_PHONEME_TYPE | phNASAL },
+	{ "isLiquid",     0, CONDITION_IS_PHONEME_TYPE | phLIQUID },
+	{ "isUStop",      0, CONDITION_IS_PHONEME_TYPE | phSTOP },
+	{ "isVStop",      0, CONDITION_IS_PHONEME_TYPE | phVSTOP },
+	{ "isVFricative", 0, CONDITION_IS_PHONEME_TYPE | phVFRICATIVE },
 
-	{ "isPalatal",  0, i_isPalatal },
-	{ "isLong",     0, i_isLong },
-	{ "isRhotic",   0, i_isRhotic },
-	{ "isSibilant", 0, i_isSibilant },
-	{ "isFlag1",    0, i_isFlag1 },
-	{ "isFlag2",    0, i_isFlag2 },
-	{ "isFlag3",    0, i_isFlag3 },
+	{ "isPalatal",  0, CONDITION_IS_PHFLAG_SET | phFLAGBIT_PALATAL },
+	{ "isLong",     0, CONDITION_IS_PHFLAG_SET | phFLAGBIT_LONG },
+	{ "isRhotic",   0, CONDITION_IS_PHFLAG_SET | phFLAGBIT_RHOTIC },
+	{ "isSibilant", 0, CONDITION_IS_PHFLAG_SET | phFLAGBIT_SIBILANT },
+	{ "isFlag1",    0, CONDITION_IS_PHFLAG_SET | phFLAGBIT_FLAG1 },
+	{ "isFlag2",    0, CONDITION_IS_PHFLAG_SET | phFLAGBIT_FLAG2 },
 
-	{ "isVel", 0, i_isVel },
+	{ "isVelar", 0, CONDITION_IS_PLACE_OF_ARTICULATION | phPLACE_VELAR },
 
-	{ "isDiminished",  0, i_isDiminished },
-	{ "isUnstressed",  0, i_isUnstressed },
-	{ "isNotStressed", 0, i_isNotStressed },
-	{ "isStressed",    0, i_isStressed },
-	{ "isMaxStress",   0, i_isMaxStress },
+	{ "isDiminished",  0, CONDITION_IS_OTHER | STRESS_IS_DIMINISHED },
+	{ "isUnstressed",  0, CONDITION_IS_OTHER | STRESS_IS_UNSTRESSED },
+	{ "isNotStressed", 0, CONDITION_IS_OTHER | STRESS_IS_NOT_STRESSED },
+	{ "isStressed",    0, CONDITION_IS_OTHER | STRESS_IS_SECONDARY },
+	{ "isMaxStress",   0, CONDITION_IS_OTHER | STRESS_IS_PRIMARY },
 
-	{ "isBreak",            0, i_isBreak },
-	{ "isPause2",           0, i_isBreak },  // synonym for isBreak
-	{ "isWordStart",        0, i_isWordStart },
-	{ "notWordStart",       0, i_notWordStart },
-	{ "isWordEnd",          0, i_isWordEnd },
-	{ "isAfterStress",      0, i_isAfterStress },
-	{ "isNotVowel",         0, i_isNotVowel },
-	{ "isFinalVowel",       0, i_isFinalVowel },
-	{ "isVoiced",           0, i_isVoiced },   // voiced consonant, or vowel
-	{ "isFirstVowel",       0, i_isFirstVowel },
-	{ "isSecondVowel",      0, i_isSecondVowel },
-	{ "isSeqFlag1",         0, i_isSeqFlag1 },
-	{ "isTranslationGiven", 0, i_IsTranslationGiven },   // phoneme translation given in **_list or as [[...]]
+	{ "isPause2",           0, CONDITION_IS_OTHER | isBreak },
+	{ "isWordStart",        0, CONDITION_IS_OTHER | isWordStart },
+	{ "isWordEnd",          0, CONDITION_IS_OTHER | isWordEnd },
+	{ "isAfterStress",      0, CONDITION_IS_OTHER | isAfterStress },
+	{ "isNotVowel",         0, CONDITION_IS_OTHER | isNotVowel },
+	{ "isFinalVowel",       0, CONDITION_IS_OTHER | isFinalVowel },
+	{ "isVoiced",           0, CONDITION_IS_OTHER | isVoiced },
+	{ "isFirstVowel",       0, CONDITION_IS_OTHER | isFirstVowel },
+	{ "isSecondVowel",      0, CONDITION_IS_OTHER | isSecondVowel },
+	{ "isTranslationGiven", 0, CONDITION_IS_OTHER | isTranslationGiven },
 
 	{ NULL, 0, 0 }
 };
@@ -151,8 +151,6 @@ enum {
 	kPROCEDURE,
 	kENDPHONEME,
 	kENDPROCEDURE,
-	kEQUIVALENTS,
-	kPHONEMENUMBER,
 	kPHONEMETABLE,
 	kINCLUDE,
 	kIMPORT_PH,
@@ -219,27 +217,13 @@ static keywtab_t k_intonation[] = {
 };
 
 static keywtab_t keywords[] = {
-	{ "vowel",   tPHONEME_TYPE, phVOWEL },
 	{ "liquid",  tPHONEME_TYPE, phLIQUID },
 	{ "pause",   tPHONEME_TYPE, phPAUSE },
 	{ "stress",  tPHONEME_TYPE, phSTRESS },
 	{ "virtual", tPHONEME_TYPE, phVIRTUAL },
-
-	{ "fricative",      tPHONEME_TYPE, phFRICATIVE },
-	{ "vstop",          tPHONEME_TYPE, phVSTOP },
-	{ "vfricative",     tPHONEME_TYPE, phVFRICATIVE },
 	{ "delete_phoneme", tPHONEME_TYPE, phDELETED },
 
-	// type of consonant
-	{ "stop",  tPHONEME_TYPE, phSTOP },
-	{ "frc",   tPHONEME_TYPE, phFRICATIVE },
-	{ "nasal", tPHONEME_TYPE, phNASAL },
-	{ "flp",   tPHONEME_TYPE, phVSTOP },
-	{ "afr",   tPHONEME_TYPE, phSTOP },      // treat as stop
-	{ "apr",   tPHONEME_TYPE, phFRICATIVE }, // [h] voiceless approximant
-
 	// keywords
-	{ "phonemenumber",        tSTATEMENT, kPHONEMENUMBER },
 	{ "phonemetable",         tSTATEMENT, kPHONEMETABLE },
 	{ "include",              tSTATEMENT, kINCLUDE },
 	{ (const char *)utf8_bom, tSTATEMENT, kUTF8_BOM },
@@ -248,7 +232,6 @@ static keywtab_t keywords[] = {
 	{ "procedure",      tSTATEMENT, kPROCEDURE },
 	{ "endphoneme",     tSTATEMENT, kENDPHONEME },
 	{ "endprocedure",   tSTATEMENT, kENDPROCEDURE },
-	{ "equivalents",    tSTATEMENT, kEQUIVALENTS },
 	{ "import_phoneme", tSTATEMENT, kIMPORT_PH },
 	{ "stress_type",    tSTATEMENT, kSTRESSTYPE },
 	{ "starttype",      tSTATEMENT, kSTARTTYPE },
@@ -283,60 +266,31 @@ static keywtab_t keywords[] = {
 	{ "InsertPhoneme",       tINSTRN1, i_INSERT_PHONEME },
 	{ "AppendPhoneme",       tINSTRN1, i_APPEND_PHONEME },
 	{ "IfNextVowelAppend",   tINSTRN1, i_APPEND_IFNEXTVOWEL },
-	{ "ChangeIfDiminished",  tINSTRN1, i_CHANGE_IF + 0 },
-	{ "ChangeIfUnstressed",  tINSTRN1, i_CHANGE_IF + 1 },
-	{ "ChangeIfNotStressed", tINSTRN1, i_CHANGE_IF + 2 },
-	{ "ChangeIfStressed",    tINSTRN1, i_CHANGE_IF + 3 },
+	{ "ChangeIfDiminished",  tINSTRN1, i_CHANGE_IF | STRESS_IS_DIMINISHED },
+	{ "ChangeIfUnstressed",  tINSTRN1, i_CHANGE_IF | STRESS_IS_UNSTRESSED },
+	{ "ChangeIfNotStressed", tINSTRN1, i_CHANGE_IF | STRESS_IS_NOT_STRESSED },
+	{ "ChangeIfStressed",    tINSTRN1, i_CHANGE_IF | STRESS_IS_SECONDARY },
+	{ "ChangeIfStressed",    tINSTRN1, i_CHANGE_IF | STRESS_IS_PRIMARY },
 
 	{ "PauseBefore", tINSTRN1, i_PAUSE_BEFORE },
 	{ "PauseAfter",  tINSTRN1, i_PAUSE_AFTER },
-	{ "Length",      tINSTRN1, i_SET_LENGTH },
 	{ "length",      tINSTRN1, i_SET_LENGTH },
 	{ "LongLength",  tINSTRN1, i_LONG_LENGTH },
 	{ "LengthAdd",   tINSTRN1, i_ADD_LENGTH },
-	{ "Lengthmod",   tINSTRN1, i_LENGTH_MOD },
 	{ "lengthmod",   tINSTRN1, i_LENGTH_MOD },
 	{ "ipa",         tINSTRN1, i_IPA_NAME },
 
 	// flags
-	{ "wavef",        tPHONEME_FLAG, phWAVE },
 	{ "unstressed",   tPHONEME_FLAG, phUNSTRESSED },
-	{ "fortis",       tPHONEME_FLAG, phFORTIS },
-	{ "sibilant",     tPHONEME_FLAG, phSIBILANT },
 	{ "nolink",       tPHONEME_FLAG, phNOLINK },
-	{ "trill",        tPHONEME_FLAG, phTRILL },
-	{ "vowel2",       tPHONEME_FLAG, phVOWEL2 },
-	{ "palatal",      tPHONEME_FLAG, phPALATAL },
-	{ "long",         tPHONEME_FLAG, phLONG },
-	{ "dontlist",     tPHONEME_FLAG, phDONTLIST },
 	{ "brkafter",     tPHONEME_FLAG, phBRKAFTER },
 	{ "rhotic",       tPHONEME_FLAG, phRHOTIC },
-	{ "nonsyllabic",  tPHONEME_FLAG, phNONSYLLABIC },
 	{ "lengthenstop", tPHONEME_FLAG, phLENGTHENSTOP },
 	{ "nopause",      tPHONEME_FLAG, phNOPAUSE },
 	{ "prevoice",     tPHONEME_FLAG, phPREVOICE },
 
 	{ "flag1", tPHONEME_FLAG, phFLAG1 },
 	{ "flag2", tPHONEME_FLAG, phFLAG2 },
-	{ "flag3", tPHONEME_FLAG, phFLAG3 },
-
-	// voiced / unvoiced
-	{ "vcd", tPHONEME_FLAG, phVOICED },
-	{ "vls", tPHONEME_FLAG, phFORTIS },
-
-	// place of articulation, set bits 16-19 of phflags
-	{ "blb", tPLACE,  1 },
-	{ "lbd", tPLACE,  2 },
-	{ "dnt", tPLACE,  3 },
-	{ "alv", tPLACE,  4 },
-	{ "rfx", tPLACE,  5 },
-	{ "pla", tPLACE,  6 },
-	{ "pal", tPLACE,  7 },
-	{ "vel", tPLACE,  8 },
-	{ "lbv", tPLACE,  9 },
-	{ "uvl", tPLACE, 10 },
-	{ "phr", tPLACE, 11 },
-	{ "glt", tPLACE, 12 },
 
 	// vowel transition attributes
 	{ "len=",   tTRANSITION,  1 },
@@ -366,20 +320,16 @@ static int n_phcodes_list[N_PHONEME_TABS];
 static PHONEME_TAB_LIST phoneme_tab_list2[N_PHONEME_TABS];
 static PHONEME_TAB *phoneme_tab2;
 static int phoneme_flags;
-static int place_articulation;
-
-static char *p_equivalence;
-static char equivalence_buf[20000];
 
 #define N_PROCS 50
 int n_procs;
 int proc_addr[N_PROCS];
-char proc_names[40][N_PROCS];
+char proc_names[N_ITEM_STRING+1][N_PROCS];
 
 #define MAX_PROG_BUF 2000
-USHORT *prog_out;
-USHORT *prog_out_max;
-USHORT prog_buf[MAX_PROG_BUF+20];
+unsigned short *prog_out;
+unsigned short *prog_out_max;
+unsigned short prog_buf[MAX_PROG_BUF+20];
 
 static espeak_ng_STATUS ReadPhondataManifest(espeak_ng_ERROR_CONTEXT *context)
 {
@@ -462,7 +412,7 @@ static int error_count = 0;
 static int resample_count = 0;
 static int resample_fails = 0;
 static int then_count = 0;
-static int after_if = 0;
+static bool after_if = false;
 
 static char current_fname[80];
 
@@ -496,9 +446,9 @@ STACK stack[N_STACK];
 #define N_IF_STACK 12
 int if_level;
 typedef struct {
-	USHORT *p_then;
-	USHORT *p_else;
-	int returned;
+	unsigned short *p_then;
+	unsigned short *p_else;
+	bool returned;
 } IF_STACK;
 IF_STACK if_stack[N_IF_STACK];
 
@@ -517,7 +467,6 @@ enum {
 
 int item_type;
 int item_terminator;
-#define N_ITEM_STRING 256
 char item_string[N_ITEM_STRING];
 
 static int ref_sorter(char **a, char **b)
@@ -550,7 +499,7 @@ static void CompileReport(void)
 	int procedure_num;
 	int prev_mnemonic;
 
-	if (f_report == NULL)
+	if (f_report == NULL || count_references == 0)
 		return;
 
 	// make a list of all the references and sort it
@@ -626,6 +575,16 @@ static void error(const char *format, ...)
 	va_end(args);
 }
 
+static void error_from_status(espeak_ng_STATUS status, const char *context)
+{
+	char message[512];
+	espeak_ng_GetStatusCodeMessage(status, message, sizeof(message));
+	if (context)
+		error("%s: '%s'.", message, context);
+	else
+		error("%s.", message);
+}
+
 static unsigned int StringToWord(const char *string)
 {
 	// Pack 4 characters into a word
@@ -769,7 +728,7 @@ static void unget_char(unsigned int c)
 		linenum--;
 }
 
-int CheckNextChar()
+static int CheckNextChar()
 {
 	int c;
 	while (((c = get_char()) == ' ') || (c == '\t'))
@@ -837,8 +796,7 @@ static int NextItem(int type)
 
 	if ((c == ')') || (c == ','))
 		c = ' ';
-
-	if (!feof(f_in))
+	else if (!feof(f_in))
 		unget_char(c);
 
 	if (type == tSTRING)
@@ -940,7 +898,7 @@ static int Range(int value, int divide, int min, int max)
 	return value - min;
 }
 
-int CompileVowelTransition(int which)
+static int CompileVowelTransition(int which)
 {
 	// Compile a vowel transition
 	int key;
@@ -1046,7 +1004,7 @@ int CompileVowelTransition(int which)
 	return 0;
 }
 
-espeak_ng_STATUS LoadSpect(const char *path, int control, int *addr)
+static espeak_ng_STATUS LoadSpect(const char *path, int control, int *addr)
 {
 	SpectSeq *spectseq;
 	int peak;
@@ -1242,9 +1200,9 @@ static int LoadWavefile(FILE *f, const char *fname)
 	int max = 0;
 	int length;
 	int sr1, sr2;
-	int failed;
+	bool failed;
 	int len;
-	int resample_wav = 0;
+	bool resample_wav = false;
 	const char *fname2;
 	char fname_temp[100];
 	char msg[120];
@@ -1259,7 +1217,7 @@ static int LoadWavefile(FILE *f, const char *fname)
 		int fd_temp;
 		char command[sizeof(path_home)+250];
 
-		failed = 0;
+		failed = false;
 
 #ifdef HAVE_MKSTEMP
 		strcpy(fname_temp, "/tmp/espeakXXXXXX");
@@ -1279,7 +1237,7 @@ static int LoadWavefile(FILE *f, const char *fname)
 
 		sprintf(command, "sox \"%s/%s.wav\" -r %d -c1 -t wav %s\n", phsrc, fname2, samplerate_native, fname_temp);
 		if (system(command) != 0)
-			failed = 1;
+			failed = true;
 
 		if (failed || (GetFileLength(fname_temp) <= 0)) {
 			if (resample_fails < 2)
@@ -1302,7 +1260,7 @@ static int LoadWavefile(FILE *f, const char *fname)
 		if (f_report != NULL)
 			fprintf(f_report, "resampled  %s\n", fname);
 		resample_count++;
-		resample_wav = 1;
+		resample_wav = true;
 		fseek(f, 40, SEEK_SET); // skip past the WAV header, up to before "data length"
 	}
 
@@ -1374,7 +1332,7 @@ static int LoadWavefile(FILE *f, const char *fname)
 		length++;
 	}
 
-	if (resample_wav != 0) {
+	if (resample_wav == true) {
 		fclose(f);
 		remove(fname_temp);
 	}
@@ -1436,27 +1394,30 @@ static int LoadEnvelope2(FILE *f, const char *fname)
 	unsigned char env[ENV_LEN];
 
 	n_points = 0;
-	fgets(line_buf, sizeof(line_buf), f); // skip first line
-	while (!feof(f)) {
-		if (fgets(line_buf, sizeof(line_buf), f) == NULL)
-			break;
+	if (fgets(line_buf, sizeof(line_buf), f) != NULL) { ; // skip first line, then loop
+		while (!feof(f)) {
+			if (fgets(line_buf, sizeof(line_buf), f) == NULL)
+				break;
 
-		env_lin[n_points] = 0;
-		n = sscanf(line_buf, "%f %f %d", &env_x[n_points], &env_y[n_points], &env_lin[n_points]);
-		if (n >= 2) {
-			env_x[n_points] *= (float)1.28; // convert range 0-100 to 0-128
-			n_points++;
+			env_lin[n_points] = 0;
+			n = sscanf(line_buf, "%f %f %d", &env_x[n_points], &env_y[n_points], &env_lin[n_points]);
+			if (n >= 2) {
+				env_x[n_points] *= (float)1.28; // convert range 0-100 to 0-128
+				n_points++;
+			}
 		}
 	}
-	env_x[n_points] = env_x[n_points-1];
-	env_y[n_points] = env_y[n_points-1];
+	if (n_points > 0) {
+		env_x[n_points] = env_x[n_points-1];
+		env_y[n_points] = env_y[n_points-1];
+	}
 
 	ix = -1;
 	ix2 = 0;
-	for (x = 0; x < ENV_LEN; x++) {
-		if (x > env_x[ix+4])
+	if (n_points > 0) for (x = 0; x < ENV_LEN; x++) {
+		if (n_points > 3 && x > env_x[ix+4])
 			ix++;
-		if (x >= env_x[ix2+1])
+		if (n_points > 2 && x >= env_x[ix2+1])
 			ix2++;
 
 		if (env_lin[ix2] > 0) {
@@ -1478,7 +1439,7 @@ static int LoadEnvelope2(FILE *f, const char *fname)
 	}
 
 	displ = ftell(f_phdata);
-	fwrite(env, 1, 128, f_phdata);
+	fwrite(env, 1, ENV_LEN, f_phdata);
 
 	return displ;
 }
@@ -1659,23 +1620,23 @@ static void CompileSound(int keyword, int isvowel)
    =8         data = stress bitmap
    =9         special tests
  */
-int CompileIf(int elif)
+static int CompileIf(int elif)
 {
 	int key;
-	int finish = 0;
+	bool finish = false;
 	int word = 0;
 	int word2;
 	int data;
 	int bitmap;
 	int brackets;
-	int not_flag;
-	USHORT *prog_last_if = NULL;
+	bool not_flag;
+	unsigned short *prog_last_if = NULL;
 
 	then_count = 2;
-	after_if = 1;
+	after_if = true;
 
 	while (!finish) {
-		not_flag = 0;
+		not_flag = false;
 		word2 = 0;
 		if (prog_out >= prog_out_max) {
 			error("Phoneme program too large");
@@ -1686,7 +1647,7 @@ int CompileIf(int elif)
 			error("Expected a condition, not '%s'", item_string);
 
 		if ((item_type == 0) && (key == k_NOT)) {
-			not_flag = 1;
+			not_flag = true;
 			if ((key = NextItem(tCONDITION)) < 0)
 				error("Expected a condition, not '%s'", item_string);
 		}
@@ -1748,7 +1709,7 @@ int CompileIf(int elif)
 				*prog_last_if |=  i_OR;
 			break;
 		case k_THEN:
-			finish = 1;
+			finish = true;
 			break;
 		default:
 			error("Expected AND, OR, THEN");
@@ -1761,16 +1722,16 @@ int CompileIf(int elif)
 		if_stack[if_level].p_else = NULL;
 	}
 
-	if_stack[if_level].returned = 0;
+	if_stack[if_level].returned = false;
 	if_stack[if_level].p_then = prog_out;
 	*prog_out++ = i_JUMP_FALSE;
 
 	return 0;
 }
 
-void FillThen(int add)
+static void FillThen(int add)
 {
-	USHORT *p;
+	unsigned short *p;
 	int offset;
 
 	p = if_stack[if_level].p_then;
@@ -1796,22 +1757,22 @@ void FillThen(int add)
 	then_count = 0;
 }
 
-int CompileElse(void)
+static int CompileElse(void)
 {
-	USHORT *ref;
-	USHORT *p;
+	unsigned short *ref;
+	unsigned short *p;
 
 	if (if_level < 1) {
 		error("ELSE not expected");
 		return 0;
 	}
 
-	if (if_stack[if_level].returned == 0)
+	if (if_stack[if_level].returned == false)
 		FillThen(1);
 	else
 		FillThen(0);
 
-	if (if_stack[if_level].returned == 0) {
+	if (if_stack[if_level].returned == false) {
 		ref = prog_out;
 		*prog_out++ = 0;
 
@@ -1823,7 +1784,7 @@ int CompileElse(void)
 	return 0;
 }
 
-int CompileElif(void)
+static int CompileElif(void)
 {
 	if (if_level < 1) {
 		error("ELIF not expected");
@@ -1835,9 +1796,9 @@ int CompileElif(void)
 	return 0;
 }
 
-int CompileEndif(void)
+static int CompileEndif(void)
 {
-	USHORT *p;
+	unsigned short *p;
 	int chain;
 	int offset;
 
@@ -1934,8 +1895,19 @@ static void ImportPhoneme(void)
 
 	NextItem(tSTRING);
 
-	if ((ph = FindPhoneme(item_string)) == NULL)
+	if ((ph = FindPhoneme(item_string)) == NULL) {
+		error("Cannot find phoneme '%s' to import.", item_string);
 		return;
+	}
+
+	if (phoneme_out->phflags != 0 ||
+	    phoneme_out->type != phINVALID ||
+	    phoneme_out->start_type != 0 ||
+	    phoneme_out->end_type != 0 ||
+	    phoneme_out->std_length != 0 ||
+	    phoneme_out->length_mod != 0) {
+		error("Phoneme import will override set properties.");
+	}
 
 	ph_mnem = phoneme_out->mnemonic;
 	ph_code = phoneme_out->code;
@@ -1970,12 +1942,12 @@ static void CallPhoneme(void)
 		if (phoneme_out->type == phINVALID) {
 			// Phoneme type has not been set. Copy it from the called phoneme
 			phoneme_out->type = ph->type;
-			phoneme_flags = ph->phflags & ~phARTICULATION;
-			place_articulation = (ph->phflags & phARTICULATION) >> 16;
 			phoneme_out->start_type = ph->start_type;
 			phoneme_out->end_type = ph->end_type;
 			phoneme_out->std_length = ph->std_length;
 			phoneme_out->length_mod = ph->length_mod;
+
+			phoneme_flags = ph->phflags & ~phARTICULATION;
 		}
 	}
 
@@ -1989,15 +1961,7 @@ static void DecThenCount()
 		then_count--;
 }
 
-static void InstnPlusPhoneme(int instn)
-{
-	int phcode;
-
-	phcode = NextItemBrackets(tPHONEMEMNEM, 0);
-	*prog_out++ = instn + phcode;
-}
-
-int CompilePhoneme(int compile_phoneme)
+static int CompilePhoneme(int compile_phoneme)
 {
 	int endphoneme = 0;
 	int keyword;
@@ -2018,10 +1982,9 @@ int CompilePhoneme(int compile_phoneme)
 	prog_out = prog_buf;
 	prog_out_max = &prog_buf[MAX_PROG_BUF-1];
 	if_level = 0;
-	if_stack[0].returned = 0;
-	after_if = 0;
+	if_stack[0].returned = false;
+	after_if = false;
 	phoneme_flags = 0;
-	place_articulation = 0;
 
 	NextItem(tSTRING);
 	if (compile_phoneme) {
@@ -2047,6 +2010,7 @@ int CompilePhoneme(int compile_phoneme)
 	phoneme_out->start_type = 0;
 	phoneme_out->end_type = 0;
 	phoneme_out->length_mod = 0;
+	phoneme_out->phflags = 0;
 
 	while (!endphoneme && !feof(f_in)) {
 		if ((keyword = NextItem(tKEYWORD)) < 0) {
@@ -2054,22 +2018,26 @@ int CompilePhoneme(int compile_phoneme)
 				error("Missing 'endphoneme' before end-of-file"); // end of file
 				break;
 			}
-			error("Bad keyword in phoneme definition '%s'", item_string);
+
+			phoneme_feature_t feature = phoneme_feature_from_string(item_string);
+			espeak_ng_STATUS status = phoneme_add_feature(phoneme_out, feature);
+			if (status == ENS_OK)
+				continue;
+			error_from_status(status, item_string);
 			continue;
 		}
 
 		switch (item_type)
 		{
 		case tPHONEME_TYPE:
-			if (phoneme_out->type != phINVALID)
-				error("More than one phoneme type: %s", item_string);
+			if (phoneme_out->type != phINVALID) {
+				if (phoneme_out->type == phFRICATIVE && keyword == phLIQUID)
+					; // apr liquid => ok
+				else
+					error("More than one phoneme type: %s", item_string);
+			}
 			phoneme_out->type = keyword;
 			break;
-		case tPLACE:
-			if (place_articulation > 0)
-				error("Place of articulation has already been given: %s", item_string);
-			place_articulation = keyword;
-			break;
 		case tPHONEME_FLAG:
 			phoneme_flags |= keyword;
 			break;
@@ -2083,11 +2051,13 @@ int CompilePhoneme(int compile_phoneme)
 			case i_INSERT_PHONEME:
 			case i_REPLACE_NEXT_PHONEME:
 			case i_VOICING_SWITCH:
-			case i_CHANGE_IF+0:
-			case i_CHANGE_IF+1:
-			case i_CHANGE_IF+2:
-			case i_CHANGE_IF+3:
-				InstnPlusPhoneme(keyword << 8);
+			case i_CHANGE_IF | STRESS_IS_DIMINISHED:
+			case i_CHANGE_IF | STRESS_IS_UNSTRESSED:
+			case i_CHANGE_IF | STRESS_IS_NOT_STRESSED:
+			case i_CHANGE_IF | STRESS_IS_SECONDARY:
+			case i_CHANGE_IF | STRESS_IS_PRIMARY:
+				value = NextItemBrackets(tPHONEMEMNEM, 0);
+				*prog_out++ = (keyword << 8) + value;
 				DecThenCount();
 				break;
 			case i_PAUSE_BEFORE:
@@ -2105,7 +2075,7 @@ int CompilePhoneme(int compile_phoneme)
 				if (phoneme_out->type == phVOWEL)
 					value = (value * vowel_length_factor)/100;
 
-				if (after_if == 0)
+				if (after_if == false)
 					phoneme_out->std_length = value/2;
 				else {
 					*prog_out++ = (i_SET_LENGTH << 8) + value/2;
@@ -2187,12 +2157,16 @@ int CompilePhoneme(int compile_phoneme)
 				if (phcode == -1)
 					phcode = LookupPhoneme(item_string, 1);
 				phoneme_out->start_type = phcode;
+				if (phoneme_out->type == phINVALID)
+					error("a phoneme type or manner of articulation must be specified before starttype");
 				break;
 			case kENDTYPE:
 				phcode = NextItem(tPHONEMEMNEM);
 				if (phcode == -1)
 					phcode = LookupPhoneme(item_string, 1);
-				if (phoneme_out->type == phVOWEL)
+				if (phoneme_out->type == phINVALID)
+					error("a phoneme type or manner of articulation must be specified before endtype");
+				else if (phoneme_out->type == phVOWEL)
 					phoneme_out->end_type = phcode;
 				else if (phcode != phoneme_out->start_type)
 					error("endtype must equal starttype for consonants");
@@ -2201,7 +2175,10 @@ int CompilePhoneme(int compile_phoneme)
 				phcode = NextItem(tPHONEMEMNEM);
 				if (phcode == -1)
 					phcode = LookupPhoneme(item_string, 1);
-				phoneme_out->end_type = phcode; // use end_type field for consonants as voicing_switch
+				if (phoneme_out->type == phVOWEL)
+					error("voicingswitch cannot be used on vowels");
+				else
+					phoneme_out->end_type = phcode; // use end_type field for consonants as voicing_switch
 				break;
 			case kSTRESSTYPE:
 				value = NextItem(tNUMBER);
@@ -2236,7 +2213,7 @@ int CompilePhoneme(int compile_phoneme)
 				DecThenCount();
 				break;
 			case kFMT:
-				if_stack[if_level].returned = 1;
+				if_stack[if_level].returned = true;
 				DecThenCount();
 				if (phoneme_out->type == phVOWEL)
 					CompileSound(keyword, 1);
@@ -2244,7 +2221,7 @@ int CompilePhoneme(int compile_phoneme)
 					CompileSound(keyword, 0);
 				break;
 			case kWAV:
-				if_stack[if_level].returned = 1;
+				if_stack[if_level].returned = true;
 				// fallthrough:
 			case kVOWELSTART:
 			case kVOWELENDING:
@@ -2265,15 +2242,14 @@ int CompilePhoneme(int compile_phoneme)
 				CompileToneSpec();
 				break;
 			case kCONTINUE:
-				*prog_out++ = i_CONTINUE;
+				*prog_out++ = INSTN_CONTINUE;
 				DecThenCount();
 				break;
 			case kRETURN:
-				*prog_out++ = i_RETURN;
+				*prog_out++ = INSTN_RETURN;
 				DecThenCount();
 				break;
 			case kINCLUDE:
-			case kPHONEMENUMBER:
 			case kPHONEMETABLE:
 				error("Missing 'endphoneme' before '%s'", item_string);  // drop through to endphoneme
 				// fallthrough:
@@ -2282,8 +2258,8 @@ int CompilePhoneme(int compile_phoneme)
 				endphoneme = 1;
 				if (if_level > 0)
 					error("Missing ENDIF");
-				if ((prog_out > prog_buf) && (if_stack[0].returned == 0))
-					*prog_out++ = i_RETURN;
+				if ((prog_out > prog_buf) && (if_stack[0].returned == false))
+					*prog_out++ = INSTN_RETURN;
 				break;
 			}
 			break;
@@ -2298,7 +2274,6 @@ int CompilePhoneme(int compile_phoneme)
 			error("Phoneme type is missing");
 			phoneme_out->type = 0;
 		}
-		phoneme_out->phflags = place_articulation << 16;
 		phoneme_out->phflags |= phoneme_flags;
 
 		if (phoneme_out->phflags & phVOICED) {
@@ -2322,7 +2297,7 @@ int CompilePhoneme(int compile_phoneme)
 	if (prog_out > prog_buf) {
 		// write out the program for this phoneme
 		fflush(f_phindex);
-		phoneme_out->program = ftell(f_phindex) / sizeof(USHORT);
+		phoneme_out->program = ftell(f_phindex) / sizeof(unsigned short);
 
 		if (f_prog_log != NULL) {
 			phoneme_prog_log.addr = phoneme_out->program;
@@ -2331,8 +2306,8 @@ int CompilePhoneme(int compile_phoneme)
 		}
 
 		if (compile_phoneme == 0)
-			proc_addr[n_procs++] =  ftell(f_phindex) / sizeof(USHORT);
-		fwrite(prog_buf, sizeof(USHORT), prog_out - prog_buf, f_phindex);
+			proc_addr[n_procs++] =  ftell(f_phindex) / sizeof(unsigned short);
+		fwrite(prog_buf, sizeof(unsigned short), prog_out - prog_buf, f_phindex);
 	}
 
 	return 0;
@@ -2374,7 +2349,6 @@ static void WritePhonemeTables()
 		fputc(phoneme_tab_list2[ix].includes, f_phtab);
 		fputc(0, f_phtab);
 		fputc(0, f_phtab);
-		Write4Bytes(f_phtab, phoneme_tab_list2[ix].equivalence_tables); // byte index into phondata for equivalence tables
 
 		fwrite(phoneme_tab_list2[ix].name, 1, N_PHONEME_TAB_NAME, f_phtab);
 
@@ -2394,8 +2368,6 @@ static void WritePhonemeTables()
 static void EndPhonemeTable()
 {
 	int ix;
-	int *pw;
-	int length;
 
 	if (n_phoneme_tabs == 0)
 		return;
@@ -2411,18 +2383,6 @@ static void EndPhonemeTable()
 	}
 
 	n_phcodes_list[n_phoneme_tabs-1] = n_phcodes;
-
-	if ((length = p_equivalence - equivalence_buf) > 0) {
-		// terminate the list of phoneme equivalence tables
-		pw = (int *)p_equivalence;
-		pw[0] = 0;
-
-		// write the equivalence data into phondata, and remember it's address
-		ix = ftell(f_phdata);
-		fprintf(f_phcontents, "Q  0x%.5x  %s\n", ix, phoneme_tab_list2[n_phoneme_tabs-1].name);
-		phoneme_tab_list2[n_phoneme_tabs-1].equivalence_tables = ix;
-		fwrite(equivalence_buf, length+4, 1, f_phdata);
-	}
 }
 
 static void StartPhonemeTable(const char *name)
@@ -2448,7 +2408,6 @@ static void StartPhonemeTable(const char *name)
 	strncpy0(phoneme_tab_list2[n_phoneme_tabs].name, name, N_PHONEME_TAB_NAME);
 	n_phcodes = 1;
 	phoneme_tab_list2[n_phoneme_tabs].includes = 0;
-	p_equivalence = equivalence_buf;
 
 	if (n_phoneme_tabs > 0) {
 		NextItem(tSTRING); // name of base phoneme table
@@ -2466,7 +2425,7 @@ static void StartPhonemeTable(const char *name)
 				break;
 			}
 		}
-		if (ix == n_phoneme_tabs)
+		if (ix == n_phoneme_tabs && strcmp(item_string, "_") != 0)
 			error("Can't find base phonemetable '%s'", item_string);
 	} else
 		ReservePhCodes();
@@ -2474,97 +2433,6 @@ static void StartPhonemeTable(const char *name)
 	n_phoneme_tabs++;
 }
 
-static void CompileEquivalents()
-{
-	// a list of phonemes in another language and the equivalent phoneme strings in this language
-
-	int ix;
-	int n_names;
-	int n_bytes;
-	int foreign_error = 0;
-	int remove_stress = 0;
-	char *p_start;
-	char *p;
-	int foreign_table;
-	char foreign_table_name[40];
-	char line_buf[80];
-	char names[6][80];
-	char phcode[7];
-
-	NextItem(tSTRING);
-	strcpy(foreign_table_name, item_string);
-
-	if ((foreign_table = SelectPhonemeTableName(foreign_table_name)) < 0) {
-		if (strcmp(foreign_table_name, "NULL") != 0)
-			error("Unknown phoneme table '%s'", foreign_table_name);
-		foreign_error = 1;
-	}
-
-	p_start = p_equivalence;
-	p_equivalence += 8;
-
-	p_start[0] = foreign_table;
-
-	linenum--;
-	while (!feof(f_in)) {
-		linenum++;
-		if (fgets(line_buf, sizeof(line_buf), f_in) == NULL)
-			break;
-
-		if ((p = strstr(line_buf, "//")) != NULL)
-			*p = 0;
-
-		for (ix = 0; ix < 6; ix++)
-			names[ix][0] = 0;
-		n_names = sscanf(line_buf, "%s %s %s %s %s %s", names[0], names[1], names[2], names[3], names[4], names[5]);
-		if (n_names < 1)
-			continue;
-
-		if (strcmp(names[0], "endphoneme") == 0)
-			break;
-
-		if (foreign_error)
-			continue;
-
-		if (strcmp(names[0], "remove_stress") == 0) {
-			remove_stress = 1;
-			continue;
-		}
-
-		if (p_equivalence > &equivalence_buf[sizeof(equivalence_buf) - 16]) {
-			error("'equivalents' tables are too large");
-			break;
-		}
-
-		if (foreign_error == 0) {
-			phcode[0] = LookupPhonemeString(names[0]);
-			if (phcode[0] == 0) {
-				sprintf(line_buf, "%s/%s", foreign_table_name, names[0]);
-				error("Unknown phoneme '%s'", line_buf);
-			}
-		}
-
-		for (ix = 1; ix < n_names; ix++)
-			phcode[ix] = LookupPhoneme(names[ix], 1);
-
-		// only write a translation if it has an effect
-		if ((n_names > 2) || (phcode[0] != phcode[1])) {
-			// write: foreign phoneme number, then a string of local phoneme numbers
-			memcpy(p_equivalence, phcode, n_names);
-			p_equivalence += n_names;
-			*p_equivalence++ = 0;
-		}
-	}
-	*p_equivalence++ = 0;
-
-	p_equivalence = (char *)((intptr_t)(p_equivalence + 3) & ~0x3); // align to word boundary
-	n_bytes = p_equivalence - p_start;
-	p_start[1] = remove_stress;
-	n_bytes = n_bytes / 4;
-	p_start[2] = n_bytes >> 8; // index of next table
-	p_start[3] = n_bytes;
-}
-
 static void CompilePhonemeFiles()
 {
 	int item;
@@ -2626,9 +2494,6 @@ static void CompilePhonemeFiles()
 		case kPROCEDURE:
 			CompilePhoneme(0);
 			break;
-		case kEQUIVALENTS:
-			CompileEquivalents();
-			break;
 		default:
 			if (!feof(f_in))
 				error("Keyword 'phoneme' expected");
@@ -2716,7 +2581,6 @@ espeak_ng_CompilePhonemeDataPath(long rate,
 	        "#   S - A SPECT_SEQ structure\n"
 	        "#   W - A wavefile segment\n"
 	        "#   E - An envelope\n"
-	        "#   Q - Phoneme equivalence tables\n"
 	        "#\n"
 	        "# Address is the displacement within phondata of this item\n"
 	        "#\n"
@@ -2842,7 +2706,7 @@ MNEM_TAB envelope_names[] = {
 	{ NULL, -1 }
 };
 
-int LookupEnvelopeName(const char *name)
+static int LookupEnvelopeName(const char *name)
 {
 	return LookupMnem(envelope_names, name);
 }
@@ -2858,11 +2722,11 @@ espeak_ng_STATUS espeak_ng_CompileIntonation(FILE *log, espeak_ng_ERROR_CONTEXT
 	char c;
 	int keyword;
 	int n_tune_names = 0;
-	int done_split = 0;
-	int done_onset = 0;
-	int done_last = 0;
+	bool done_split = false;
+	bool done_onset = false;
+	bool done_last = false;
 	int n_preset_tunes = 0;
-	int found;
+	int found = 0;
 	int tune_number = 0;
 	FILE *f_out;
 	TUNE *tune_data;
@@ -2946,7 +2810,7 @@ espeak_ng_STATUS espeak_ng_CompileIntonation(FILE *log, espeak_ng_ERROR_CONTEXT
 		switch (keyword)
 		{
 		case kTUNE:
-			done_split = 0;
+			done_split = false;
 
 			memcpy(&new_tune, &default_tune, sizeof(TUNE));
 			NextItem(tSTRING);
@@ -2970,11 +2834,12 @@ espeak_ng_STATUS espeak_ng_CompileIntonation(FILE *log, espeak_ng_ERROR_CONTEXT
 				error("Bad tune name: '%s;", new_tune.name);
 			break;
 		case kENDTUNE:
-			if (done_onset == 0) {
+			if (!found) continue;
+			if (done_onset == false) {
 				new_tune.unstr_start[0] = new_tune.unstr_start[1];
 				new_tune.unstr_end[0] = new_tune.unstr_end[1];
 			}
-			if (done_last == 0) {
+			if (done_last == false) {
 				new_tune.unstr_start[2] = new_tune.unstr_start[1];
 				new_tune.unstr_end[2] = new_tune.unstr_end[1];
 			}
@@ -2988,13 +2853,13 @@ espeak_ng_STATUS espeak_ng_CompileIntonation(FILE *log, espeak_ng_ERROR_CONTEXT
 			new_tune.onset = NextItem(tNUMBER);
 			new_tune.unstr_start[0] = NextItem(tSIGNEDNUMBER);
 			new_tune.unstr_end[0] = NextItem(tSIGNEDNUMBER);
-			done_onset = 1;
+			done_onset = true;
 			break;
 		case kTUNE_HEADLAST:
 			new_tune.head_last = NextItem(tNUMBER);
 			new_tune.unstr_start[2] = NextItem(tSIGNEDNUMBER);
 			new_tune.unstr_end[2] = NextItem(tSIGNEDNUMBER);
-			done_last = 1;
+			done_last = true;
 			break;
 		case kTUNE_HEADENV:
 			NextItem(tSTRING);
@@ -3058,7 +2923,7 @@ espeak_ng_STATUS espeak_ng_CompileIntonation(FILE *log, espeak_ng_ERROR_CONTEXT
 				error("Bad envelope name: '%s'", item_string);
 				break;
 			}
-			done_split = 1;
+			done_split = true;
 			new_tune.split_nucleus_env = ix;
 			new_tune.split_nucleus_max = NextItem(tNUMBER);
 			new_tune.split_nucleus_min = NextItem(tNUMBER);
