@@ -31,14 +31,16 @@
 #include <espeak-ng/speak_lib.h>
 #include <espeak-ng/encoding.h>
 
+#include "readclause.h"
+#include "synthdata.h"
+
 #include "error.h"
 #include "speech.h"
 #include "phoneme.h"
-#include "synthesize.h"
 #include "voice.h"
+#include "synthesize.h"
 #include "translate.h"
 
-const char *version_string = PACKAGE_VERSION;
 const int version_phdata  = 0x014801;
 
 // copy the current phoneme table into here
@@ -47,7 +49,7 @@ int current_phoneme_table;
 PHONEME_TAB *phoneme_tab[N_PHONEME_TAB];
 unsigned char phoneme_tab_flags[N_PHONEME_TAB];   // bit 0: not inherited
 
-USHORT *phoneme_index = NULL;
+unsigned short *phoneme_index = NULL;
 char *phondata_ptr = NULL;
 unsigned char *wavefile_data = NULL;
 static unsigned char *phoneme_tab_data = NULL;
@@ -58,15 +60,9 @@ int phoneme_tab_number = 0;
 
 int wavefile_ix; // a wavefile to play along with the synthesis
 int wavefile_amp;
-int wavefile_ix2;
-int wavefile_amp2;
 
 int seq_len_adjust;
 int vowel_transition[4];
-int vowel_transition0;
-int vowel_transition1;
-
-int FormantTransition2(frameref_t *seq, int *n_frames, unsigned int data1, unsigned int data2, PHONEME_TAB *other_ph, int which);
 
 static espeak_ng_STATUS ReadPhFile(void **ptr, const char *fname, int *size, espeak_ng_ERROR_CONTEXT *context)
 {
@@ -112,7 +108,6 @@ espeak_ng_STATUS LoadPhData(int *srate, espeak_ng_ERROR_CONTEXT *context)
 	int length = 0;
 	int rate;
 	unsigned char *p;
-	int *pw;
 
 	espeak_ng_STATUS status;
 	if ((status = ReadPhFile((void **)&phoneme_tab_data, "phontab", NULL, context)) != ENS_OK)
@@ -336,19 +331,19 @@ unsigned char *GetEnvelope(int index)
 	return (unsigned char *)&phondata_ptr[index];
 }
 
-static void SetUpPhonemeTable(int number, int recursing)
+static void SetUpPhonemeTable(int number, bool recursing)
 {
 	int ix;
 	int includes;
 	int ph_code;
 	PHONEME_TAB *phtab;
 
-	if (recursing == 0)
+	if (recursing == false)
 		memset(phoneme_tab_flags, 0, sizeof(phoneme_tab_flags));
 
 	if ((includes = phoneme_tab_list[number].includes) > 0) {
 		// recursively include base phoneme tables
-		SetUpPhonemeTable(includes-1, 1);
+		SetUpPhonemeTable(includes-1, true);
 	}
 
 	// now add the phonemes from this table
@@ -367,7 +362,7 @@ static void SetUpPhonemeTable(int number, int recursing)
 void SelectPhonemeTable(int number)
 {
 	n_phoneme_tab = 0;
-	SetUpPhonemeTable(number, 0); // recursively for included phoneme tables
+	SetUpPhonemeTable(number, false); // recursively for included phoneme tables
 	n_phoneme_tab++;
 	current_phoneme_table = number;
 }
@@ -469,15 +464,15 @@ static bool StressCondition(Translator *tr, PHONEME_LIST *plist, int condition,
 
 		if ((tr->langopts.param[LOPT_REDUCE] & 0x2) && (stress_level >= pl->wordstress)) {
 			// treat the most stressed syllable in an unstressed word as stressed
-			stress_level = 4;
+			stress_level = STRESS_IS_PRIMARY;
 		}
 	}
 
-	if (condition == isMaxStress)
+	if (condition == STRESS_IS_PRIMARY)
 		return stress_level >= pl->wordstress;
 
-	if (condition == isStressed) {
-		if (stress_level > 3)
+	if (condition == STRESS_IS_SECONDARY) {
+		if (stress_level > STRESS_IS_SECONDARY)
 			return true;
 	} else {
 		if (stress_level < condition_level[condition])
@@ -501,15 +496,14 @@ static int CountVowelPosition(PHONEME_LIST *plist)
 	return count;
 }
 
-static bool InterpretCondition(Translator *tr, int control, PHONEME_LIST *plist, USHORT *p_prog, WORD_PH_DATA *worddata)
+static bool InterpretCondition(Translator *tr, int control, PHONEME_LIST *plist, unsigned short *p_prog, WORD_PH_DATA *worddata)
 {
 	int which;
 	int ix;
 	unsigned int data;
 	int instn;
 	int instn2;
-	int count;
-	int check_endtype = 0;
+	bool check_endtype = false;
 	PHONEME_TAB *ph;
 	PHONEME_LIST *plist_this;
 
@@ -556,7 +550,7 @@ static bool InterpretCondition(Translator *tr, int control, PHONEME_LIST *plist,
 		case 0: // prevPh
 		case 5: // prevPhW
 			plist--;
-			check_endtype = 1;
+			check_endtype = true;
 			break;
 		case 1: // thisPh
 			break;
@@ -583,7 +577,7 @@ static bool InterpretCondition(Translator *tr, int control, PHONEME_LIST *plist,
 			if ((worddata == NULL) || (worddata->prev_vowel.ph == NULL))
 				return false; // no previous vowel
 			plist = &(worddata->prev_vowel);
-			check_endtype = 1;
+			check_endtype = true;
 			break;
 		case 9: // next3PhW
 			for (ix = 1; ix <= 3; ix++) {
@@ -596,7 +590,7 @@ static bool InterpretCondition(Translator *tr, int control, PHONEME_LIST *plist,
 			if ((plist[0].sourceix) || (plist[-1].sourceix))
 				return false;
 			plist -= 2;
-			check_endtype = 1;
+			check_endtype = true;
 			break;
 		}
 
@@ -637,11 +631,11 @@ static bool InterpretCondition(Translator *tr, int control, PHONEME_LIST *plist,
 		case CONDITION_IS_OTHER:
 			switch (data)
 			{
-			case isDiminished:
-			case isUnstressed:
-			case isNotStressed:
-			case isStressed:
-			case isMaxStress:
+			case STRESS_IS_DIMINISHED:
+			case STRESS_IS_UNSTRESSED:
+			case STRESS_IS_NOT_STRESSED:
+			case STRESS_IS_SECONDARY:
+			case STRESS_IS_PRIMARY:
 				return StressCondition(tr, plist, data, 0);
 			case isBreak:
 				return (ph->type == phPAUSE) || (plist_this->synthflags & SFLAG_NEXT_PAUSE);
@@ -697,9 +691,9 @@ static bool InterpretCondition(Translator *tr, int control, PHONEME_LIST *plist,
 	return false;
 }
 
-static void SwitchOnVowelType(PHONEME_LIST *plist, PHONEME_DATA *phdata, USHORT **p_prog, int instn_type)
+static void SwitchOnVowelType(PHONEME_LIST *plist, PHONEME_DATA *phdata, unsigned short **p_prog, int instn_type)
 {
-	USHORT *prog;
+	unsigned short *prog;
 	int voweltype;
 	signed char x;
 
@@ -720,7 +714,7 @@ static void SwitchOnVowelType(PHONEME_LIST *plist, PHONEME_DATA *phdata, USHORT
 	*p_prog += 12;
 }
 
-int NumInstnWords(USHORT *prog)
+int NumInstnWords(unsigned short *prog)
 {
 	int instn;
 	int instn2;
@@ -760,7 +754,7 @@ int NumInstnWords(USHORT *prog)
 			// This instruction is followed by addWav(), 2 more words
 			return 4;
 		}
-		if (instn2 == OPCODE_CONTINUE)
+		if (instn2 == INSTN_CONTINUE)
 			return 3;
 		return 2;
 	}
@@ -773,8 +767,8 @@ void InterpretPhoneme(Translator *tr, int control, PHONEME_LIST *plist, PHONEME_
 	// bit 8:  change phonemes
 
 	PHONEME_TAB *ph;
-	USHORT *prog;
-	USHORT instn;
+	unsigned short *prog;
+	unsigned short instn;
 	int instn2;
 	int or_flag;
 	bool truth;
@@ -786,7 +780,7 @@ void InterpretPhoneme(Translator *tr, int control, PHONEME_LIST *plist, PHONEME_
 
 	#define N_RETURN 10
 	int n_return = 0;
-	USHORT *return_addr[N_RETURN]; // return address stack
+	unsigned short *return_addr[N_RETURN]; // return address stack
 
 	ph = plist->ph;
 
@@ -817,10 +811,10 @@ void InterpretPhoneme(Translator *tr, int control, PHONEME_LIST *plist, PHONEME_
 				// instructions with no operand
 				switch (data)
 				{
-				case OPCODE_RETURN:
+				case INSTN_RETURN:
 					end_flag = 1;
 					break;
-				case OPCODE_CONTINUE:
+				case INSTN_CONTINUE:
 					break;
 				default:
 					InvalidInstn(ph, instn);
@@ -958,7 +952,7 @@ void InterpretPhoneme(Translator *tr, int control, PHONEME_LIST *plist, PHONEME_
 			param_sc = phdata->sound_param[instn2] = (instn >> 4) & 0xff;
 			prog++;
 
-			if (prog[1] != OPCODE_CONTINUE) {
+			if (prog[1] != INSTN_CONTINUE) {
 				if (instn2 < 2) {
 					// FMT() and WAV() imply Return
 					end_flag = 1;
