@@ -20,6 +20,7 @@
 #include "config.h"
 
 #include <ctype.h>
+#include <stdbool.h>
 #include <stdint.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -31,10 +32,16 @@
 #include <espeak-ng/speak_lib.h>
 #include <espeak-ng/encoding.h>
 
+#include "dictionary.h"
+#include "numbers.h"
+#include "phonemelist.h"
+#include "readclause.h"
+#include "synthdata.h"
+
 #include "speech.h"
 #include "phoneme.h"
-#include "synthesize.h"
 #include "voice.h"
+#include "synthesize.h"
 #include "translate.h"
 
 Translator *translator = NULL; // the main translator
@@ -42,7 +49,6 @@ Translator *translator2 = NULL; // secondary translator for certain words
 static char translator2_language[20] = { 0 };
 
 FILE *f_trans = NULL; // phoneme output text
-int option_tone2 = 0;
 int option_tone_flags = 0; // bit 8=emphasize allcaps, bit 9=emphasize penultimate stress
 int option_phonemes = 0;
 int option_phoneme_events = 0;
@@ -54,7 +60,6 @@ static int option_sayas2 = 0; // used in translate_clause()
 static int option_emphasis = 0; // 0=normal, 1=normal, 2=weak, 3=moderate, 4=strong
 int option_ssml = 0;
 int option_phoneme_input = 0; // allow [[phonemes]] in input
-int option_phoneme_variants = 0; // 0= don't display phoneme variant mnemonics
 int option_wordgap = 0;
 
 static int count_sayas_digits;
@@ -62,19 +67,19 @@ int skip_sentences;
 int skip_words;
 int skip_characters;
 char skip_marker[N_MARKER_LENGTH];
-int skipping_text; // waiting until word count, sentence count, or named marker is reached
+bool skipping_text; // waiting until word count, sentence count, or named marker is reached
 int end_character_position;
 int count_sentences;
 int count_words;
 int clause_start_char;
 int clause_start_word;
-int new_sentence;
+bool new_sentence;
 static int word_emphasis = 0; // set if emphasis level 3 or 4
 static int embedded_flag = 0; // there are embedded commands to be applied to the next phoneme, used in TranslateWord2()
 
 static int prev_clause_pause = 0;
 static int max_clause_pause = 0;
-static int any_stressed_words;
+static bool any_stressed_words;
 int pre_pause;
 ALPHABET *current_alphabet;
 
@@ -254,7 +259,7 @@ int IsDigit(unsigned int c)
 	return 0;
 }
 
-int IsSpace(unsigned int c)
+static int IsSpace(unsigned int c)
 {
 	if (c == 0)
 		return 0;
@@ -265,8 +270,20 @@ int IsSpace(unsigned int c)
 	return iswspace(c);
 }
 
+int isspace2(unsigned int c)
+{
+	// can't use isspace() because on Windows, isspace(0xe1) gives TRUE !
+	int c2;
+
+	if (((c2 = (c & 0xff)) == 0) || (c > ' '))
+		return 0;
+	return 1;
+}
+
 void DeleteTranslator(Translator *tr)
 {
+	if (!tr) return;
+
 	if (tr->data_dictlist != NULL)
 		free(tr->data_dictlist);
 	free(tr);
@@ -365,7 +382,7 @@ int utf8_in(int *c, const char *buf)
 	/* Read a unicode characater from a UTF8 string
 	 * Returns the number of UTF8 bytes used.
 	 * buf: position of buffer is moved, if character is read
-	 * c: holds integer representation of multibyte character by
+	 * c: holds UTF-16 representation of multibyte character by
 	 * skipping UTF-8 header bits of bytes in following way:
 	 * 2-byte character "ā":
 	 * hex            binary
@@ -378,7 +395,7 @@ int utf8_in(int *c, const char *buf)
 	 *            1010  011001  000101
 	 *    |       +  +--.\   \  |    |
 	 *    V        `--.  \`.  `.|    |
-	 *   A645         0001001101000101
+	 *   A645         1010011001000101
 	 * 4-byte character "𠜎":
 	 * f0a09c8e 11110000101000001001110010001110
 	 *    V          000  100000  011100  001110
@@ -388,6 +405,40 @@ int utf8_in(int *c, const char *buf)
 }
 #pragma GCC visibility pop
 
+int utf8_out(unsigned int c, char *buf)
+{
+	// write a UTF-16 character into a buffer as UTF-8
+	// returns the number of bytes written
+
+	int n_bytes;
+	int j;
+	int shift;
+	static char unsigned code[4] = { 0, 0xc0, 0xe0, 0xf0 };
+
+	if (c < 0x80) {
+		buf[0] = c;
+		return 1;
+	}
+	if (c >= 0x110000) {
+		buf[0] = ' '; // out of range character code
+		return 1;
+	}
+	if (c < 0x0800)
+		n_bytes = 1;
+	else if (c < 0x10000)
+		n_bytes = 2;
+	else
+		n_bytes = 3;
+
+	shift = 6*n_bytes;
+	buf[0] = code[n_bytes] | (c >> shift);
+	for (j = 0; j < n_bytes; j++) {
+		shift -= 6;
+		buf[j+1] = 0x80 + ((c >> shift) & 0x3f);
+	}
+	return n_bytes+1;
+}
+
 char *strchr_w(const char *s, int c)
 {
 	// return NULL for any non-ascii character
@@ -396,22 +447,11 @@ char *strchr_w(const char *s, int c)
 	return strchr((char *)s, c); // (char *) is needed for Borland compiler
 }
 
-int IsAllUpper(const char *word)
-{
-	int c;
-	while ((*word != 0) && !isspace2(*word)) {
-		word += utf8_in(&c, word);
-		if (!iswupper(c))
-			return 0;
-	}
-	return 1;
-}
-
 static char *SpeakIndividualLetters(Translator *tr, char *word, char *phonemes, int spell_word)
 {
 	int posn = 0;
 	int capitals = 0;
-	int non_initial = 0;
+	bool non_initial = false;
 
 	if (spell_word > 2)
 		capitals = 2; // speak 'capital'
@@ -419,9 +459,9 @@ static char *SpeakIndividualLetters(Translator *tr, char *word, char *phonemes,
 		capitals |= 4; // speak charater code for unknown letters
 
 	while ((*word != ' ') && (*word != 0)) {
-		word += TranslateLetter(tr, word, phonemes, capitals | non_initial);
+		word += TranslateLetter(tr, word, phonemes, capitals | non_initial, current_alphabet);
 		posn++;
-		non_initial = 1;
+		non_initial = true;
 		if (phonemes[0] == phonSWITCH) {
 			// change to another language in order to translate this word
 			strcpy(word_phonemes, phonemes);
@@ -487,8 +527,6 @@ static int CheckDottedAbbrev(char *word1)
 	return count;
 }
 
-extern char *phondata_ptr;
-
 static int TranslateWord3(Translator *tr, char *word_start, WORD_TAB *wtab, char *word_out)
 {
 	// word1 is terminated by space (0x20) character
@@ -516,19 +554,17 @@ static int TranslateWord3(Translator *tr, char *word_start, WORD_TAB *wtab, char
 	char word_copy2[N_WORD_BYTES];
 	int word_copy_length;
 	char prefix_chars[0x3f + 2];
-	int found = 0;
+	bool found = false;
 	int end_flags;
 	int c_temp; // save a character byte while we temporarily replace it with space
 	int first_char;
 	int last_char = 0;
 	int prefix_flags = 0;
-	int more_suffixes;
-	int confirm_prefix;
+	bool more_suffixes;
+	bool confirm_prefix;
 	int spell_word;
-	int stress_bits;
 	int emphasize_allcaps = 0;
 	int wflags;
-	int wmark;
 	int was_unpronouncable = 0;
 	int loopcount;
 	int add_suffix_phonemes = 0;
@@ -545,7 +581,6 @@ static int TranslateWord3(Translator *tr, char *word_start, WORD_TAB *wtab, char
 		wtab = wtab_null;
 	}
 	wflags = wtab->flags;
-	wmark = wtab->wmark;
 
 	dictionary_flags[0] = 0;
 	dictionary_flags[1] = 0;
@@ -618,7 +653,7 @@ static int TranslateWord3(Translator *tr, char *word_start, WORD_TAB *wtab, char
 				strcpy(word_out, word1);
 
 			return dictionary_flags[0];
-		} else if ((found == 0) && (dictionary_flags[0] & FLAG_SKIPWORDS) && !(dictionary_flags[0] & FLAG_ABBREV)) {
+		} else if ((found == false) && (dictionary_flags[0] & FLAG_SKIPWORDS) && !(dictionary_flags[0] & FLAG_ABBREV)) {
 			// grouped words, but no translation.  Join the words with hyphens.
 			wordx = word1;
 			ix = 0;
@@ -648,11 +683,6 @@ static int TranslateWord3(Translator *tr, char *word_start, WORD_TAB *wtab, char
 			return 0;
 		}
 
-		if ((wmark > 0) && (wmark < 8)) {
-			// the stressed syllable has been specified in the text  (TESTING)
-			dictionary_flags[0] = (dictionary_flags[0] & ~0xf) | wmark;
-		}
-
 		if (!found && (dictionary_flags[0] & FLAG_ABBREV)) {
 			// the word has $abbrev flag, but no pronunciation specified.  Speak as individual letters
 			spell_word = 1;
@@ -709,16 +739,15 @@ static int TranslateWord3(Translator *tr, char *word_start, WORD_TAB *wtab, char
 		if (wflags & FLAG_TRANSLATOR2)
 			return 0;
 		return dictionary_flags[0] & FLAG_SKIPWORDS; // for "b.c.d"
-	} else if (found == 0) {
+	} else if (found == false) {
 		// word's pronunciation is not given in the dictionary list, although
 		// dictionary_flags may have ben set there
 
 		int posn;
-		int non_initial;
+		bool non_initial = false;
 		int length;
 
 		posn = 0;
-		non_initial = 0;
 		length = 999;
 		wordx = word1;
 
@@ -732,9 +761,9 @@ static int TranslateWord3(Translator *tr, char *word_start, WORD_TAB *wtab, char
 				break;
 
 			if (posn > 0)
-				non_initial = 1;
+				non_initial = true;
 
-			wordx += TranslateLetter(tr, wordx, unpron_phonemes, non_initial);
+			wordx += TranslateLetter(tr, wordx, unpron_phonemes, non_initial, current_alphabet);
 			posn++;
 			if (unpron_phonemes[0] == phonSWITCH) {
 				// change to another language in order to translate this word
@@ -780,8 +809,8 @@ static int TranslateWord3(Translator *tr, char *word_start, WORD_TAB *wtab, char
 
 			c_temp = wordx[-1];
 
-			found = 0;
-			confirm_prefix = 1;
+			found = false;
+			confirm_prefix = true;
 			for (loopcount = 0; (loopcount < 50) && (end_type & SUFX_P); loopcount++) {
 				// Found a standard prefix, remove it and retranslate
 				// loopcount guards against an endless loop
@@ -807,7 +836,7 @@ static int TranslateWord3(Translator *tr, char *word_start, WORD_TAB *wtab, char
 								fprintf(f_trans, "  suffix [%s]\n\n", end_phonemes2);
 							}
 						}
-						confirm_prefix = 0;
+						confirm_prefix = false;
 						continue;
 					}
 				}
@@ -839,7 +868,7 @@ static int TranslateWord3(Translator *tr, char *word_start, WORD_TAB *wtab, char
 				}
 				c_temp = wordx[-1];
 				wordx[-1] = ' ';
-				confirm_prefix = 1;
+				confirm_prefix = true;
 				wflags |= FLAG_PREFIX_REMOVED;
 
 				if (prefix_type & SUFX_B) {
@@ -871,7 +900,7 @@ static int TranslateWord3(Translator *tr, char *word_start, WORD_TAB *wtab, char
 					dictionary_flags[1] = dictionary_flags2[1];
 				} else
 					prefix_flags = 1;
-				if (found == 0) {
+				if (found == false) {
 					end_type = TranslateRules(tr, wordx, phonemes, N_WORD_PHONEMES, end_phonemes, wflags & (FLAG_HYPHEN_AFTER | FLAG_PREFIX_REMOVED), dictionary_flags);
 
 					if (phonemes[0] == phonSWITCH) {
@@ -889,10 +918,10 @@ static int TranslateWord3(Translator *tr, char *word_start, WORD_TAB *wtab, char
 
 				// The word has a standard ending, re-translate without this ending
 				end_flags = RemoveEnding(tr, wordx, end_type, word_copy);
-				more_suffixes = 1;
+				more_suffixes = true;
 
 				while (more_suffixes) {
-					more_suffixes = 0;
+					more_suffixes = false;
 					phonemes[0] = 0;
 
 					if (prefix_phonemes[0] != 0) {
@@ -913,10 +942,10 @@ static int TranslateWord3(Translator *tr, char *word_start, WORD_TAB *wtab, char
 						if (found)
 							prefix_phonemes[0] = 0; // matched whole word, don't need prefix now
 
-						if ((found == 0) && (dictionary_flags2[0] != 0))
+						if ((found == false) && (dictionary_flags2[0] != 0))
 							prefix_flags = 1;
 					}
-					if (found == 0) {
+					if (found == false) {
 						found = LookupDictList(tr, &wordx, phonemes, dictionary_flags2, end_flags, wtab);  // without prefix and suffix
 						if (phonemes[0] == phonSWITCH) {
 							// change to another language in order to translate this word
@@ -930,7 +959,7 @@ static int TranslateWord3(Translator *tr, char *word_start, WORD_TAB *wtab, char
 							dictionary_flags[1] = dictionary_flags2[1];
 						}
 					}
-					if (found == 0) {
+					if (found == false) {
 						if (end_type & SUFX_Q) {
 							// don't retranslate, use the original lookup result
 							strcpy(phonemes, phonemes2);
@@ -949,7 +978,7 @@ static int TranslateWord3(Translator *tr, char *word_start, WORD_TAB *wtab, char
 								if ((end_type != 0) && !(end_type & SUFX_P)) {
 									// there is another suffix
 									end_flags = RemoveEnding(tr, wordx, end_type, NULL);
-									more_suffixes = 1;
+									more_suffixes = true;
 								}
 							} else {
 								// don't remove any previous suffix
@@ -1130,7 +1159,7 @@ int TranslateWord(Translator *tr, char *word_start, WORD_TAB *wtab, char *word_o
 	char words_phonemes[N_WORD_PHONEMES]; // a word translated into phoneme codes
 	char *phonemes = words_phonemes;
 	int available = N_WORD_PHONEMES;
-	int first_word = 1;
+	bool first_word = true;
 
 	int flags = TranslateWord3(tr, word_start, wtab, word_out);
 	if (flags & FLAG_TEXTMODE && word_out) {
@@ -1139,7 +1168,7 @@ int TranslateWord(Translator *tr, char *word_start, WORD_TAB *wtab, char *word_o
 		char word[N_WORD_BYTES+1];
 		word[0] = 0;
 		word[1] = ' ';
-		memcpy(word+2, word_out, strlen(word_out));
+		strcpy(word+2, word_out);
 		word_out = word+2;
 
 		while (*word_out && available > 1) {
@@ -1157,7 +1186,7 @@ int TranslateWord(Translator *tr, char *word_start, WORD_TAB *wtab, char *word_o
 			int n;
 			if (first_word) {
 				n = snprintf(phonemes, available, "%s", word_phonemes);
-				first_word = 0;
+				first_word = false;
 			} else {
 				n = snprintf(phonemes, available, "%c%s", phonEND_WORD, word_phonemes);
 			}
@@ -1165,11 +1194,25 @@ int TranslateWord(Translator *tr, char *word_start, WORD_TAB *wtab, char *word_o
 			available -= n;
 			phonemes += n;
 
-			// skip to the next word in a multi-word replacement
-			while (!isspace(*word_out)) ++word_out;
-			while (isspace(*word_out))  ++word_out;
+			// skip to the next word in a multi-word replacement. Always skip at least one word.
+			for (dictionary_skipwords++; dictionary_skipwords > 0; dictionary_skipwords--) {
+				while (!isspace(*word_out)) ++word_out;
+				while (isspace(*word_out))  ++word_out;
+			}
+		}
+
+		// If the list file contains a text replacement to another
+		// entry in the list file, e.g.:
+		//     ripost     riposte $text
+		//     riposte    rI#p0st
+		// calling it from a prefix or suffix rule such as 'riposted'
+		// causes word_out[0] to be NULL, as TranslateWord3 has the
+		// information needed to perform the mapping. In this case,
+		// no phonemes have been written in this loop and the phonemes
+		// have been calculated, so don't override them.
+		if (phonemes != words_phonemes) {
+			snprintf(word_phonemes, sizeof(word_phonemes), "%s", words_phonemes);
 		}
-		snprintf(word_phonemes, sizeof(word_phonemes), "%s", words_phonemes);
 	}
 	return flags;
 }
@@ -1195,7 +1238,7 @@ static int CountSyllables(unsigned char *phonemes)
 	return count;
 }
 
-void Word_EmbeddedCmd()
+static void Word_EmbeddedCmd()
 {
 	// Process embedded commands for emphasis, sayas, and break
 	int embedded_cmd;
@@ -1272,7 +1315,7 @@ static int TranslateWord2(Translator *tr, char *word, WORD_TAB *wtab, int pre_pa
 	int prev_vowel = -1;
 	int pitch_raised = 0;
 	int switch_phonemes = -1;
-	int first_phoneme = 1;
+	bool first_phoneme = true;
 	int source_ix;
 	int len;
 	int ix;
@@ -1379,7 +1422,7 @@ static int TranslateWord2(Translator *tr, char *word, WORD_TAB *wtab, int pre_pa
 
 		if ((flags & FLAG_COMBINE) && !(wtab[1].flags & FLAG_PHONEMES)) {
 			char *p2;
-			int ok = 1;
+			bool ok = true;
 			unsigned int flags2[2];
 			int c_word2;
 			char ph_buf[N_WORD_PHONEMES];
@@ -1394,27 +1437,27 @@ static int TranslateWord2(Translator *tr, char *word, WORD_TAB *wtab, int pre_pa
 
 			utf8_in(&c_word2, p2+1); // first character of the next word;
 			if (!iswalpha(c_word2))
-				ok = 0;
+				ok = false;
 
-			if (ok != 0) {
+			if (ok == true) {
 				strcpy(ph_buf, word_phonemes);
 
 				flags2[0] = TranslateWord(translator, p2+1, wtab+1, NULL);
 				if ((flags2[0] & FLAG_WAS_UNPRONOUNCABLE) || (word_phonemes[0] == phonSWITCH))
-					ok = 0;
+					ok = false;
 
 				if (sylimit & 0x100) {
 					// only if the second word has $alt attribute
 					if ((flags2[0] & FLAG_ALT_TRANS) == 0)
-						ok = 0;
+						ok = false;
 				}
 
 				if ((sylimit & 0x200) && ((wtab+1)->flags & FLAG_LAST_WORD)) {
 					// not if the next word is end-of-sentence
-					ok = 0;
+					ok = false;
 				}
 
-				if (ok == 0)
+				if (ok == false)
 					strcpy(word_phonemes, ph_buf);
 			}
 
@@ -1568,6 +1611,10 @@ static int TranslateWord2(Translator *tr, char *word, WORD_TAB *wtab, int pre_pa
 
 		// Add the phonemes to the first stage phoneme list (ph_list2)
 		ph = phoneme_tab[ph_code];
+		if (ph == NULL) {
+			printf("Invalid phoneme code %d\n", ph_code);
+			continue;
+		}
 
 		if (ph_code == phonSWITCH) {
 			ph_list2[n_ph_list2].phcode = ph_code;
@@ -1618,7 +1665,7 @@ static int TranslateWord2(Translator *tr, char *word, WORD_TAB *wtab, int pre_pa
 				next_stress = 1; // default is 'unstressed'
 
 				if (stress >= 4)
-					any_stressed_words = 1;
+					any_stressed_words = true;
 
 				if ((prev_vowel >= 0) && (n_ph_list2-1) != prev_vowel)
 					ph_list2[n_ph_list2-1].stresslevel = stress; // set stress for previous consonant
@@ -1647,7 +1694,7 @@ static int TranslateWord2(Translator *tr, char *word, WORD_TAB *wtab, int pre_pa
 
 			ph_list2[n_ph_list2].stresslevel = stress;
 			n_ph_list2++;
-			first_phoneme = 0;
+			first_phoneme = false;
 		}
 	}
 
@@ -1743,57 +1790,91 @@ static int EmbeddedCommand(unsigned int *source_index_out)
 	return 1;
 }
 
-static int SubstituteChar(Translator *tr, unsigned int c, unsigned int next_in, int *insert, int *wordflags)
+static const char *FindReplacementChars(Translator *tr, const char **pfrom, unsigned int c, const char *next, int *ignore_next_n)
 {
-	int ix;
-	unsigned int word;
-	unsigned int new_c, c2, c_lower;
-	int upper_case = 0;
-	static int ignore_next = 0;
-	const unsigned int *replace_chars;
+	const char *from = *pfrom;
+	while ( !is_str_totally_null(from, 4) ) {
+		unsigned int fc = 0; // from character
+		unsigned int nc = c; // next character
+		const char *match_next = next;
 
-	if (ignore_next) {
-		ignore_next = 0;
+		*pfrom = from;
+
+		from += utf8_in((int *)&fc, from);
+		if (nc == fc) {
+			if (*from == 0) return from + 1;
+
+			bool matched = true;
+			int nmatched = 0;
+			while (*from != 0) {
+				from += utf8_in((int *)&fc, from);
+
+				match_next += utf8_in((int *)&nc, match_next);
+				nc = towlower2(nc, tr);
+
+				if (nc != fc)
+					matched = false;
+				else
+					nmatched++;
+			}
+
+			if (*from == 0 && matched) {
+				*ignore_next_n = nmatched;
+				return from + 1;
+			}
+		}
+
+		// replacement 'from' string (skip the remaining part, if any)
+		while (*from != '\0') from++;
+		from++;
+
+		// replacement 'to' string
+		while (*from != '\0') from++;
+		from++;
+	}
+	return NULL;
+}
+
+// handle .replace rule in xx_rules file
+static int SubstituteChar(Translator *tr, unsigned int c, unsigned int next_in, const char *next, int *insert, int *wordflags)
+{
+	unsigned int new_c, c2 = ' ', c_lower;
+	int upper_case = 0;
+
+	static int ignore_next_n = 0;
+	if (ignore_next_n > 0) {
+		ignore_next_n--;
 		return 8;
 	}
+
 	if (c == 0) return 0;
 
-	if ((replace_chars = tr->langopts.replace_chars) == NULL)
+	const char *from = (const char *)tr->langopts.replace_chars;
+	if (from == NULL)
 		return c;
 
 	// there is a list of character codes to be substituted with alternative codes
 
 	if (iswupper(c_lower = c)) {
-		c_lower = towlower2(c);
+		c_lower = towlower2(c, tr);
 		upper_case = 1;
 	}
 
-	new_c = 0;
-	for (ix = 0; (word = replace_chars[ix]) != 0; ix += 2) {
-		if (c_lower == (word & 0xffff)) {
-			if ((word >> 16) == 0) {
-				new_c = replace_chars[ix+1];
-				break;
-			}
-			if ((word >> 16) == (unsigned int)towlower2(next_in)) {
-				new_c = replace_chars[ix+1];
-				ignore_next = 1;
-				break;
-			}
-		}
-	}
-
-	if (new_c == 0)
+	const char *to = FindReplacementChars(tr, &from, c_lower, next, &ignore_next_n);
+	if (to == NULL)
 		return c; // no substitution
 
-	if (new_c & 0xffe00000) {
+	if (option_phonemes & espeakPHONEMES_TRACE)
+		fprintf(f_trans, "Replace: %s > %s\n", from, to);
+
+	to += utf8_in((int *)&new_c, to);
+	if (*to != 0) {
 		// there is a second character to be inserted
 		// don't convert the case of the second character unless the next letter is also upper case
-		c2 = new_c >> 16;
+		to += utf8_in((int *)&c2, to);
 		if (upper_case && iswupper(next_in))
 			c2 = ucd_toupper(c2);
 		*insert = c2;
-		new_c &= 0xffff;
 	}
 
 	if (upper_case)
@@ -1869,12 +1950,13 @@ static int TranslateChar(Translator *tr, char *ptr, int prev_in, unsigned int c,
 		}
 		break;
 	}
-	return SubstituteChar(tr, c, next_in, insert, wordflags);
+	// handle .replace rule in xx_rules file
+	return SubstituteChar(tr, c, next_in, ptr, insert, wordflags);
 }
 
 static const char *UCase_ga[] = { "bp", "bhf", "dt", "gc", "hA", "mb", "nd", "ng", "ts", "tA", "nA", NULL };
 
-int UpperCaseInWord(Translator *tr, char *word, int c)
+static int UpperCaseInWord(Translator *tr, char *word, int c)
 {
 	int ix;
 	int len;
@@ -1913,20 +1995,20 @@ void TranslateClause(Translator *tr, int *tone_out, char **voice_change)
 	int char_inserted = 0;
 	int clause_pause;
 	int pre_pause_add = 0;
-	int word_mark = 0;
 	int all_upper_case = FLAG_ALL_UPPER;
-	int finished;
-	int single_quoted;
-	int phoneme_mode = 0;
+	int alpha_count = 0;
+	bool finished = false;
+	bool single_quoted = false;
+	bool phoneme_mode = false;
 	int dict_flags = 0; // returned from dictionary lookup
 	int word_flags; // set here
 	int next_word_flags;
-	int new_sentence2;
+	bool new_sentence2;
 	int embedded_count = 0;
 	int letter_count = 0;
-	int space_inserted = 0;
-	int syllable_marked = 0;
-	int decimal_sep_count = 0;
+	bool space_inserted = false;
+	bool syllable_marked = false;
+	bool decimal_sep_count = false;
 	char *word;
 	char *p;
 	int j, k;
@@ -1942,7 +2024,6 @@ void TranslateClause(Translator *tr, int *tone_out, char **voice_change)
 
 	int terminator;
 	int tone;
-	int tone2;
 
 	if (tr == NULL)
 		return;
@@ -1950,7 +2031,7 @@ void TranslateClause(Translator *tr, int *tone_out, char **voice_change)
 	embedded_ix = 0;
 	embedded_read = 0;
 	pre_pause = 0;
-	any_stressed_words = 0;
+	any_stressed_words = false;
 
 	if ((clause_start_char = count_characters) < 0)
 		clause_start_char = 0;
@@ -1958,7 +2039,14 @@ void TranslateClause(Translator *tr, int *tone_out, char **voice_change)
 
 	for (ix = 0; ix < N_TR_SOURCE; ix++)
 		charix[ix] = 0;
-	terminator = ReadClause(tr, source, charix, &charix_top, N_TR_SOURCE, &tone2, voice_change_name);
+	terminator = ReadClause(tr, source, charix, &charix_top, N_TR_SOURCE, &tone, voice_change_name);
+
+	if (tone_out != NULL) {
+		if (tone == 0)
+			*tone_out = (terminator & CLAUSE_INTONATION_TYPE) >> 12; // tone type not overridden in ReadClause, use default
+		else
+			*tone_out = tone; // override tone type
+	}
 
 	charix[charix_top+1] = 0;
 	charix[charix_top+2] = 0x7fff;
@@ -1968,12 +2056,6 @@ void TranslateClause(Translator *tr, int *tone_out, char **voice_change)
 	if (terminator & CLAUSE_PAUSE_LONG)
 		clause_pause = clause_pause * 32; // pause value is *320mS not *10mS
 
-	tone = (terminator & CLAUSE_INTONATION_TYPE) >> 12;
-	if (tone2 != 0) {
-		// override the tone type
-		tone = tone2;
-	}
-
 	for (p = source; *p != 0; p++) {
 		if (!isspace2(*p))
 			break;
@@ -1988,7 +2070,7 @@ void TranslateClause(Translator *tr, int *tone_out, char **voice_change)
 		if (new_sentence)
 			terminator |= CLAUSE_TYPE_SENTENCE; // carry forward an end-of-sentence indicator
 		max_clause_pause += clause_pause;
-		new_sentence2 = 0;
+		new_sentence2 = false;
 	} else {
 		max_clause_pause = clause_pause;
 		new_sentence2 = new_sentence;
@@ -2000,7 +2082,7 @@ void TranslateClause(Translator *tr, int *tone_out, char **voice_change)
 		if (skip_sentences > 0) {
 			skip_sentences--;
 			if (skip_sentences == 0)
-				skipping_text = 0;
+				skipping_text = false;
 		}
 	}
 
@@ -2020,7 +2102,6 @@ void TranslateClause(Translator *tr, int *tone_out, char **voice_change)
 	tr->prev_dict_flags[1] = 0;
 
 	word_count = 0;
-	single_quoted = 0;
 	word_flags = 0;
 	next_word_flags = 0;
 
@@ -2032,7 +2113,6 @@ void TranslateClause(Translator *tr, int *tone_out, char **voice_change)
 
 	words[0].start = ix;
 	words[0].flags = 0;
-	finished = 0;
 
 	for (j = 0; charix[j] <= 0; j++) ;
 	words[0].sourceix = charix[j];
@@ -2044,7 +2124,7 @@ void TranslateClause(Translator *tr, int *tone_out, char **voice_change)
 	}
 	words[0].length = k;
 
-	while (!finished && (ix < (int)sizeof(sbuf)) && (n_ph_list2 < N_PHONEME_LIST-4)) {
+	while (!finished && (ix < (int)sizeof(sbuf) - 1) && (n_ph_list2 < N_PHONEME_LIST-4)) {
 		prev_out2 = prev_out;
 		utf8_in2(&prev_out, &sbuf[ix-1], 1);
 
@@ -2071,7 +2151,7 @@ void TranslateClause(Translator *tr, int *tone_out, char **voice_change)
 		next_in_nbytes = utf8_in(&next_in, &source[source_index]);
 
 		if (c == 0) {
-			finished = 1;
+			finished = true;
 			c = ' ';
 		}
 
@@ -2096,7 +2176,7 @@ void TranslateClause(Translator *tr, int *tone_out, char **voice_change)
 		if ((option_sayas2 == SAYAS_KEY) && (c != ' ')) {
 			if ((prev_in == ' ') && (next_in == ' '))
 				option_sayas2 = SAYAS_SINGLE_CHARS; // single character, speak its name
-			c = towlower2(c);
+			c = towlower2(c, tr);
 		}
 
 
@@ -2104,7 +2184,7 @@ void TranslateClause(Translator *tr, int *tone_out, char **voice_change)
 			all_upper_case = FLAG_PHONEMES;
 
 			if ((c == ']') && (next_in == ']')) {
-				phoneme_mode = 0;
+				phoneme_mode = false;
 				source_index++;
 				c = ' ';
 			}
@@ -2114,14 +2194,14 @@ void TranslateClause(Translator *tr, int *tone_out, char **voice_change)
 				if (count_sayas_digits > (option_sayas2 & 0xf)) {
 					// break after the specified number of digits
 					c = ' ';
-					space_inserted = 1;
+					space_inserted = true;
 					count_sayas_digits = 0;
 				}
 			} else {
 				count_sayas_digits = 0;
 				if (iswdigit(prev_out)) {
 					c = ' ';
-					space_inserted = 1;
+					space_inserted = true;
 				}
 			}
 		} else if ((option_sayas2 & 0x10) == 0) {
@@ -2145,8 +2225,8 @@ void TranslateClause(Translator *tr, int *tone_out, char **voice_change)
 				c = ' ';
 				word_flags |= FLAG_COMMA_AFTER;
 			}
-
-			c = TranslateChar(tr, &source[source_index], prev_in, c, next_in, &char_inserted, &word_flags);  // optional language specific function
+			// language specific character translations
+			c = TranslateChar(tr, &source[source_index], prev_in, c, next_in, &char_inserted, &word_flags);
 			if (c == 8)
 				continue; // ignore this character
 
@@ -2160,7 +2240,7 @@ void TranslateClause(Translator *tr, int *tone_out, char **voice_change)
 						// allow a tone number as part of the word
 					} else {
 						c = ' '; // ensure we have an end-of-word terminator
-						space_inserted = 1;
+						space_inserted = true;
 					}
 				}
 			}
@@ -2168,7 +2248,7 @@ void TranslateClause(Translator *tr, int *tone_out, char **voice_change)
 			if (iswdigit(prev_out)) {
 				if (!iswdigit(c) && (c != '.') && (c != ',') && (c != ' ')) {
 					c = ' '; // terminate digit string with a space
-					space_inserted = 1;
+					space_inserted = true;
 				}
 			} else { // Prev output is not digit
 				if (prev_in == ',') {
@@ -2177,20 +2257,21 @@ void TranslateClause(Translator *tr, int *tone_out, char **voice_change)
 					if (c == ',')
 						c = ' ';
 				} else {
-					decimal_sep_count = 0;
+					decimal_sep_count = false;
 				}
 			}
 
 			if (c == '[') {
 				if ((next_in == '\002') || ((next_in == '[') && option_phoneme_input)) {
 					//  "[\002" is used internally to start phoneme mode
-					phoneme_mode = FLAG_PHONEMES;
+					phoneme_mode = true;
 					source_index++;
 					continue;
 				}
 			}
 
 			if (IsAlpha(c)) {
+				alpha_count++;
 				if (!IsAlpha(prev_out) || (tr->langopts.ideographs && ((c > 0x3040) || (prev_out > 0x3040)))) {
 					if (wcschr(tr->punct_within_word, prev_out) == 0)
 						letter_count = 0; // don't reset count for an apostrophy within a word
@@ -2198,7 +2279,7 @@ void TranslateClause(Translator *tr, int *tone_out, char **voice_change)
 					if ((prev_out != ' ') && (wcschr(tr->punct_within_word, prev_out) == 0)) {
 						// start of word, insert space if not one there already
 						c = ' ';
-						space_inserted = 1;
+						space_inserted = true;
 
 						if (!IsBracket(prev_out)) // ?? perhaps only set FLAG_NOSPACE for . - /  (hyphenated words, URLs, etc)
 							next_word_flags |= FLAG_NOSPACE;
@@ -2225,7 +2306,7 @@ void TranslateClause(Translator *tr, int *tone_out, char **voice_change)
 							// Break into separate words
 							if (IsAlpha(prev_out)) {
 								c = ' ';
-								space_inserted = 1;
+								space_inserted = true;
 								word_flags |= FLAG_HYPHEN_AFTER;
 								next_word_flags |= FLAG_HYPHEN;
 							}
@@ -2234,23 +2315,20 @@ void TranslateClause(Translator *tr, int *tone_out, char **voice_change)
 				}
 
 				if (iswupper(c)) {
-					c = towlower2(c);
+					c = towlower2(c, tr);
 
 					if ((j = tr->langopts.param[LOPT_CAPS_IN_WORD]) > 0) {
-						if ((j == 2) && (syllable_marked == 0)) {
+						if ((j == 2) && (syllable_marked == false)) {
 							char_inserted = c;
 							c = 0x2c8; // stress marker
-							syllable_marked = 1;
+							syllable_marked = true;
 						}
 					} else {
 						if (iswlower(prev_in)) {
-							// lower case followed by upper case in a word
-							if (UpperCaseInWord(tr, &sbuf[ix], c) == 1) {
-								// convert to lower case and continue
-								c = towlower2(c);
-							} else {
-								c = ' '; // lower case followed by upper case, treat as new word
-								space_inserted = 1;
+							// lower case followed by upper case, possibly CamelCase
+							if (UpperCaseInWord(tr, &sbuf[ix], c) == 0) { // start a new word
+								c = ' ';
+								space_inserted = true;
 								prev_in_save = c;
 							}
 						} else if ((c != ' ') && iswupper(prev_in) && iswlower(next_in)) {
@@ -2262,7 +2340,7 @@ void TranslateClause(Translator *tr, int *tone_out, char **voice_change)
 							} else if (IsAlpha(next2_in)) {
 								// changing from upper to lower case, start new word at the last uppercase, if 3 or more letters
 								c = ' ';
-								space_inserted = 1;
+								space_inserted = true;
 								prev_in_save = c;
 								next_word_flags |= FLAG_NOSPACE;
 							}
@@ -2286,7 +2364,7 @@ void TranslateClause(Translator *tr, int *tone_out, char **voice_change)
 					if (prev_out != ' ') {
 						// previous 'word' not yet ended (not alpha or numeric), start new word now.
 						c = ' ';
-						space_inserted = 1;
+						space_inserted = true;
 					} else {
 						// '-' between two letters is a hyphen, treat as a space
 						word_flags |= FLAG_HYPHEN;
@@ -2312,7 +2390,7 @@ void TranslateClause(Translator *tr, int *tone_out, char **voice_change)
 				if (prev_out == '.') {
 					// multiple dots, separate by spaces. Note >3 dots has been replaced by elipsis
 					c = ' ';
-					space_inserted = 1;
+					space_inserted = true;
 				} else if ((word_count > 0) && !(words[word_count-1].flags & FLAG_NOSPACE) && IsAlpha(prev_in)) {
 					// dot after a word, with space following, probably an abbreviation
 					words[word_count-1].flags |= FLAG_HAS_DOT;
@@ -2323,25 +2401,25 @@ void TranslateClause(Translator *tr, int *tone_out, char **voice_change)
 			} else if (c == '\'') {
 				if (((prev_in == '.') || iswalnum(prev_in)) && IsAlpha(next_in)) {
 					// between two letters, or in an abbreviation (eg. u.s.a.'s). Consider the apostrophe as part of the word
-					single_quoted = 0;
+					single_quoted = false;
 				} else if ((tr->langopts.param[LOPT_APOSTROPHE] & 1) && IsAlpha(next_in))
-					single_quoted = 0; // apostrophe at start of word is part of the word
+					single_quoted = false; // apostrophe at start of word is part of the word
 				else if ((tr->langopts.param[LOPT_APOSTROPHE] & 2) && IsAlpha(prev_in))
-					single_quoted = 0; // apostrophe at end of word is part of the word
+					single_quoted = false; // apostrophe at end of word is part of the word
 				else if ((wcschr(tr->char_plus_apostrophe, prev_in) != 0) && (prev_out2 == ' ')) {
 					// consider single character plus apostrophe as a word
-					single_quoted = 0;
+					single_quoted = false;
 					if (next_in == ' ')
 						source_index++; // skip following space
 				} else {
-					if ((prev_out == 's') && (single_quoted == 0)) {
+					if ((prev_out == 's') && (single_quoted == false)) {
 						// looks like apostrophe after an 's'
 						c = ' ';
 					} else {
 						if (IsSpace(prev_out))
-							single_quoted = 1;
+							single_quoted = true;
 						else
-							single_quoted = 0;
+							single_quoted = false;
 
 						pre_pause_add = 4; // single quote
 						c = ' ';
@@ -2352,11 +2430,11 @@ void TranslateClause(Translator *tr, int *tone_out, char **voice_change)
 			else if (iswdigit(c)) {
 				if (tr->langopts.tone_numbers && IsAlpha(prev_out) && !IsDigit(next_in)) {
 				} else if ((prev_out != ' ') && !iswdigit(prev_out)) {
-					if ((prev_out != tr->langopts.decimal_sep) || ((decimal_sep_count > 0) && (tr->langopts.decimal_sep == ','))) {
+					if ((prev_out != tr->langopts.decimal_sep) || ((decimal_sep_count == true) && (tr->langopts.decimal_sep == ','))) {
 						c = ' ';
-						space_inserted = 1;
+						space_inserted = true;
 					} else
-						decimal_sep_count = 1;
+						decimal_sep_count = true;
 				} else if ((prev_out == ' ') && IsAlpha(prev_out2) && !IsAlpha(prev_in)) {
 					// insert extra space between a word and a number, to distinguish 'a 2' from 'a2'
 					sbuf[ix++] = ' ';
@@ -2398,9 +2476,11 @@ void TranslateClause(Translator *tr, int *tone_out, char **voice_change)
 					words[word_count].flags |= FLAG_EMBEDDED;
 					embedded_count = 0;
 				}
+				if (alpha_count == 0) {
+					all_upper_case &= ~FLAG_ALL_UPPER;
+				}
 				words[word_count].pre_pause = pre_pause;
 				words[word_count].flags |= (all_upper_case | word_flags | word_emphasis);
-				words[word_count].wmark = word_mark;
 
 				if (pre_pause > 0) {
 					// insert an extra space before the word, to prevent influence from previous word across the pause
@@ -2415,7 +2495,7 @@ void TranslateClause(Translator *tr, int *tone_out, char **voice_change)
 				words[word_count].start = ix;
 				words[word_count].flags = 0;
 
-				for (j = source_index; charix[j] <= 0; j++) // skip blanks
+				for (j = source_index; j < charix_top && charix[j] <= 0; j++) // skip blanks
 					;
 				words[word_count].sourceix = charix[j];
 				k = 0;
@@ -2429,15 +2509,15 @@ void TranslateClause(Translator *tr, int *tone_out, char **voice_change)
 				word_flags = next_word_flags;
 				next_word_flags = 0;
 				pre_pause = 0;
-				word_mark = 0;
 				all_upper_case = FLAG_ALL_UPPER;
-				syllable_marked = 0;
+				alpha_count = 0;
+				syllable_marked = false;
 			}
 
 			if (space_inserted) {
 				source_index = prev_source_index; // rewind to the previous character
 				char_inserted = 0;
-				space_inserted = 0;
+				space_inserted = false;
 			}
 		} else {
 			if ((ix < (N_TR_SOURCE - 4)))
@@ -2485,7 +2565,7 @@ void TranslateClause(Translator *tr, int *tone_out, char **voice_change)
 		if (skip_words > 0) {
 			skip_words--;
 			if (skip_words == 0)
-				skipping_text = 0;
+				skipping_text = false;
 		}
 		if (skipping_text)
 			continue;
@@ -2597,7 +2677,8 @@ void TranslateClause(Translator *tr, int *tone_out, char **voice_change)
 			if ((dict_flags & (FLAG_ALLOW_DOT | FLAG_NEEDS_DOT)) && (ix == word_count - 1 - dictionary_skipwords) && (terminator & CLAUSE_DOT_AFTER_LAST_WORD)) {
 				// probably an abbreviation such as Mr. or B. rather than end of sentence
 				clause_pause = 10;
-				tone = 4;
+				if (tone_out != NULL)
+					*tone_out = 4;
 			}
 		}
 
@@ -2631,7 +2712,7 @@ void TranslateClause(Translator *tr, int *tone_out, char **voice_change)
 	if (Eof() && ((word_count == 0) || (option_endpause == 0)))
 		clause_pause = 10;
 
-	MakePhonemeList(tr, clause_pause, new_sentence2);
+	MakePhonemeList(tr, clause_pause, new_sentence2, &n_ph_list2, ph_list2);
 	phoneme_list[N_PHONEME_LIST].ph = NULL; // recognize end of phoneme_list array, in Generate()
 	phoneme_list[N_PHONEME_LIST].sourceix = 1;
 
@@ -2643,12 +2724,9 @@ void TranslateClause(Translator *tr, int *tone_out, char **voice_change)
 
 	prev_clause_pause = clause_pause;
 
-	if (tone_out != NULL)
-		*tone_out = tone;
-
-	new_sentence = 0;
+	new_sentence = false;
 	if (terminator & CLAUSE_TYPE_SENTENCE)
-		new_sentence = 1; // next clause is a new sentence
+		new_sentence = true; // next clause is a new sentence
 
 	if (voice_change != NULL) {
 		// return new voice name if an embedded voice change command terminated the clause
@@ -2668,8 +2746,8 @@ void InitText(int control)
 	skip_marker[0] = 0;
 	skip_words = 0;
 	skip_characters = 0;
-	skipping_text = 0;
-	new_sentence = 1;
+	skipping_text = false;
+	new_sentence = true;
 
 	prev_clause_pause = 0;
 
