@@ -22,7 +22,7 @@
 #include <string.h>
 #include <stdio.h>
 
-void fput_utf8c(FILE *out, codepoint_t c)
+static void fput_utf8c(FILE *out, codepoint_t c)
 {
 	if (c < 0x80)
 		fputc((uint8_t)c, out);
@@ -46,7 +46,7 @@ void fput_utf8c(FILE *out, codepoint_t c)
 	}
 }
 
-int fget_utf8c(FILE *in, codepoint_t *c)
+static int fget_utf8c(FILE *in, codepoint_t *c)
 {
 	int ch = EOF;
 	if ((ch = fgetc(in)) == EOF) return 0;
@@ -79,7 +79,7 @@ int fget_utf8c(FILE *in, codepoint_t *c)
 	return 1;
 }
 
-void uprintf_codepoint(FILE *out, codepoint_t c, char mode)
+static void uprintf_codepoint(FILE *out, codepoint_t c, char mode)
 {
 	switch (mode)
 	{
@@ -101,7 +101,7 @@ void uprintf_codepoint(FILE *out, codepoint_t c, char mode)
 	}
 }
 
-void uprintf_is(FILE *out, codepoint_t c, char mode)
+static void uprintf_is(FILE *out, codepoint_t c, char mode)
 {
 	switch (mode)
 	{
@@ -144,7 +144,7 @@ void uprintf_is(FILE *out, codepoint_t c, char mode)
 	}
 }
 
-void uprintf(FILE *out, codepoint_t c, const char *format)
+static void uprintf(FILE *out, codepoint_t c, const char *format)
 {
 	while (*format) switch (*format)
 	{
@@ -210,7 +210,7 @@ void uprintf(FILE *out, codepoint_t c, const char *format)
 	}
 }
 
-void print_file(FILE *in, const char *format)
+static void print_file(FILE *in, const char *format)
 {
 	codepoint_t c = 0;
 	while (fget_utf8c(in, &c))
